/*	$Id: aux.c,v 1.16 2001/04/17 03:24:08 riq Exp $	*/
/* Tenes Empanadas Graciela
 *
 * Copyright (C) 2000 Ricardo Quesada
 *
 * Author: Ricardo Calixto Quesada <rquesada@core-sdi.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
/**
 * @file aux.c
 * Funciones auxiliares del cliente
 */

#include <string.h>

#include "client.h"

/**
 * @fn TEG_STATUS aux_status( PJUGADOR pj, char *str )
 * parsea el status de los jugadores
 */
TEG_STATUS aux_status( PJUGADOR pj, char *str )
{
	PARSER p;
	DELIM igualador={ ':', ':', ':' };
	DELIM separador={ ',', ',', ',' };

	memset( pj, 0, sizeof(*pj));

	if( strlen(str)==0 )
		goto error;

	p.igualador = &igualador;
	p.separador = &separador;
	p.data = str;

	if( parser_call( &p ) && p.hay_otro ) {
		strncpy( pj->nombre, p.token, PLAYERNAME_MAX_LEN);
	} else goto error;

	if( parser_call( &p ) && p.hay_otro ) {
		pj->color = atoi( p.token);
	} else goto error;

	if( parser_call( &p ) && p.hay_otro ) {
		pj->numjug = atoi( p.token);
	} else goto error;

	if( parser_call( &p ) && p.hay_otro ) {
		pj->estado = atoi( p.token);
	} else goto error;
	
	if( parser_call( &p ) && p.hay_otro ) {
		pj->tot_paises = atoi( p.token);
	} else goto error;

	if( parser_call( &p ) && p.hay_otro ) {
		pj->tot_ejercitos = atoi( p.token);
	} else goto error;

	if( parser_call( &p ) && p.hay_otro ) {
		pj->tot_tarjetas = atoi( p.token);
	} else goto error;

	if( parser_call( &p ) && !p.hay_otro ) {
		pj->empezo_turno = atoi( p.token);
	} else goto error;

	return TEG_STATUS_SUCCESS;
error:
	textmsg(M_ERR,"aux_status_error");
	return TEG_STATUS_ERROR;
}

/**
 * @fn TEG_STATUS aux_paises( int numjug, char *str )
 * parsea los ejercitos en paises de un jugador
 */
TEG_STATUS aux_paises( int numjug, char *str )
{
	int i,pais,cant;
	PARSER p;
	DELIM igualador={ ':', ':', ':' };
	DELIM separador={ ',', ',', ',' };

	if( strlen(str)==0 )
		goto error;

	p.igualador = &igualador;
	p.separador = &separador;
	p.data = str;

	do {
		if((i = parser_call( &p ))) {
			pais = atoi(p.token);
			cant = atoi(p.value);
			if( g_paises[pais].numjug != numjug || g_paises[pais].ejercitos != cant ) {
				g_paises[pais].numjug = numjug;
				g_paises[pais].ejercitos = cant;
				CALL_GUI(GUI_PAIS)(g_paises[pais].id);
			}
		}
	} while(i && p.hay_otro );

	return TEG_STATUS_SUCCESS;
error:
	textmsg(M_ERR,"aux_paises_error");
	return TEG_STATUS_ERROR;
}

/**
 * @fn void aux_draw_all_countries()
 * Dibuja todos los paises
 */
void aux_draw_all_countries()
{
	int i;
	for(i=0;i<PAISES_CANT;i++)
		CALL_GUI(GUI_PAIS)(i);
}

TEG_STATUS aux_jugador_init( PJUGADOR pJ, int numjug, char *name, int color )
{
	memset(pJ,0,sizeof(*pJ));

	pJ->numjug = numjug;
	strncpy( pJ->nombre, name, sizeof(pJ->nombre)-1);
	pJ->nombre[sizeof(pJ->nombre)-1]=0;
	pJ->color = color;
	pJ->estado = JUG_ESTADO_HABILITADO;
	return TEG_STATUS_SUCCESS;
}

TEG_STATUS aux_start_error()
{
	textmsg(M_ERR,_("Error in start. Are there at least 2 players?"));
	return TEG_STATUS_SUCCESS;
}
