/*	$Id: armies.c,v 1.6 2001/05/27 19:32:04 riq Exp $	*/
/* Tenes Empanadas Graciela
 *
 * Copyright (C) 2000 Ricardo Quesada
 *
 * Author: Ricardo Calixto Quesada <rquesada@core-sdi.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
/**
 * @file armies.c
 * Muestra las fichas que hay que poner
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "gui.h"
#include "client.h"
#include "interface.h"
#include "support.h"
#include "armies.h"

#define HELVETICA_10_FONT  "-adobe-helvetica-medium-r-normal-*-10-*-*-*-*-*-*-*"
#define HELVETICA_14_BFONT "-adobe-helvetica-bold-r-normal-*-14-*-*-*-*-*-*-*"

static ARMIES_POS image_pos[CONT_CANT+1] = {
	{0,0,NULL,NULL,NULL},
	{0,0,NULL,NULL,NULL},
	{0,0,NULL,NULL,NULL},
	{0,0,NULL,NULL,NULL},
	{0,0,NULL,NULL,NULL},
	{0,0,NULL,NULL,NULL},
	{0,0,NULL,NULL,NULL},
	};

static int armies_cant=0;
static int initialized=0;

static void armies_view_cant()
{
	char size_string[10];

	if( image_pos[CONT_CANT].armies_text ) {
		gtk_object_destroy(GTK_OBJECT ( image_pos[CONT_CANT].armies_text ) );
		image_pos[CONT_CANT].armies_text = NULL;
	}


	sprintf(size_string,"%d", armies_cant);
	image_pos[CONT_CANT].armies_text = gnome_canvas_item_new(
		image_pos[CONT_CANT].armies_group,
		gnome_canvas_text_get_type(),
		"text", size_string,
		"x", (double) 25,
		"y", (double) 27,
		"x_offset", (double) -1,
		"y_offset", (double) -1,
		"font", HELVETICA_14_BFONT,
		"fill_color", "black",
		"anchor",GTK_ANCHOR_CENTER,
		NULL);
}

TEG_STATUS armies_add(int pais)
{
	if(!initialized) return TEG_STATUS_ERROR;
	if( !image_pos[CONT_CANT].armies_image ) return TEG_STATUS_ERROR;

	armies_cant--;
	armies_view_cant();
	return TEG_STATUS_SUCCESS;
}

TEG_STATUS armies_del(int pais)
{
	if(!initialized) return TEG_STATUS_ERROR;
	if( !image_pos[CONT_CANT].armies_image ) return TEG_STATUS_ERROR;

	armies_cant++;
	armies_view_cant();
	return TEG_STATUS_SUCCESS;
}

TEG_STATUS armies_view_more( int cant )
{
	if(!initialized) return TEG_STATUS_ERROR;
	if( !image_pos[CONT_CANT].armies_image ) return TEG_STATUS_ERROR;

	armies_cant += cant;

	gnome_canvas_item_show( GNOME_CANVAS_ITEM(image_pos[CONT_CANT].armies_group) );

	armies_view_cant();

	return TEG_STATUS_SUCCESS;
}

TEG_STATUS armies_view(int cant, int conts)
{
	if(!initialized) return TEG_STATUS_ERROR;
	if( !image_pos[CONT_CANT].armies_image ) return TEG_STATUS_ERROR;

	armies_cant = cant + cont_tot( conts );

	gnome_canvas_item_show( GNOME_CANVAS_ITEM(image_pos[CONT_CANT].armies_group) );

	armies_view_cant();

	return TEG_STATUS_SUCCESS;
}

TEG_STATUS armies_unview()
{
	if(!initialized) return TEG_STATUS_ERROR;
	if( !image_pos[CONT_CANT].armies_image ) return TEG_STATUS_ERROR;

	armies_cant=0;
	gnome_canvas_item_hide( GNOME_CANVAS_ITEM(image_pos[CONT_CANT].armies_group) );
	return TEG_STATUS_SUCCESS;
}

TEG_STATUS armies_init( GnomeCanvasGroup *group )
{
	GdkImlibImage *box, *arrow;
	char *filename=NULL;

	if(!group)
		return TEG_STATUS_ERROR;

	if(initialized)
		return TEG_STATUS_ERROR;

	filename = load_pixmap_file( "box.png" );
	box = gdk_imlib_load_image (filename);
	if(filename) g_free( filename );

	filename = load_pixmap_file( "arrow_right.png" );
	arrow = gdk_imlib_load_image (filename);
	if(filename) g_free( filename );


	if( box==NULL || arrow==NULL ) {

		textmsg(M_ERR,_("Error, couldnt find file: %s or %s"),"box.png","arrow_right.png");
		g_warning ("Could not find file: %s or %s","box.png", "arrow_right.png");
		if(box) gdk_imlib_destroy_image( box );
		if(arrow) gdk_imlib_destroy_image( arrow );
		return TEG_STATUS_ERROR;

	} else {

		image_pos[CONT_CANT].armies_group = GNOME_CANVAS_GROUP(
			gnome_canvas_item_new(
				group,
				gnome_canvas_group_get_type (),
				"x", (float)image_pos[CONT_CANT].x,
				"y", (float)image_pos[CONT_CANT].y,
				NULL)
			);

		if(!image_pos[CONT_CANT].armies_group)
			return TEG_STATUS_ERROR;

		image_pos[CONT_CANT].armies_image = gnome_canvas_item_new(
				image_pos[CONT_CANT].armies_group,
				gnome_canvas_image_get_type (),
				"image", box,
				"x", 5.0,
				"y", 5.0,
				"width", (double) box->rgb_width,
				"height", (double) box->rgb_height,
				"anchor", GTK_ANCHOR_NW,
				NULL);

		if( image_pos[CONT_CANT].armies_image )
			gnome_canvas_item_hide( GNOME_CANVAS_ITEM(image_pos[CONT_CANT].armies_group) );
				
		gtk_signal_connect (GTK_OBJECT (image_pos[CONT_CANT].armies_image), "destroy",
				    (GtkSignalFunc) free_imlib_image,
				    box);

	};

	initialized=1;
	return TEG_STATUS_SUCCESS;
}
