/*	$Id: gui.c,v 1.75 2001/07/17 14:50:48 riq Exp $	*/
/* Tenes Empanadas Graciela
 *
 * Copyright (C) 2000 Ricardo Quesada
 *
 * Author: Ricardo Calixto Quesada <rquesada@core-sdi.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
/**
 * @file gui.c
 * gui-gnome functions
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <gnome.h>

#include "gui.h"
#include "client.h"

#include "priv.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "chatline.h"
#include "priv.h"
#include "g_pais.h"
#include "stock.h"
#include "cards.h"
#include "dices.h"
#include "status.h"
#include "objetivo.h"
#include "conectar.h"
#include "armies.h"
#include "colors.h"
#include "themes.h"


TEG_STATUS gui_init(int argc, char **argv);
TEG_STATUS gui_main(void);
TEG_STATUS gui_exit(void);
TEG_STATUS gui_disconnect(void);
TEG_STATUS gui_habilitado(int numjug);
TEG_STATUS gui_connected();
TEG_STATUS gui_status(void);
TEG_STATUS gui_start(void);
TEG_STATUS gui_fichas(int cant, int conts);
TEG_STATUS gui_turno(void);
TEG_STATUS gui_pais(int p);
TEG_STATUS gui_tropas(int src, int dst, int cant);
TEG_STATUS gui_tarjeta(int pais );
TEG_STATUS gui_dados();
TEG_STATUS gui_reagrupe( int src, int dst, int cant );
TEG_STATUS gui_sensi();
TEG_STATUS gui_canje();
TEG_STATUS gui_ataque( int src, int dst );
TEG_STATUS gui_canje();
TEG_STATUS gui_lost(int numjug);
TEG_STATUS gui_winner(int numjug, int objetivo);
TEG_STATUS gui_objetivo();
TEG_STATUS gui_textplayermsg(char *n,int nj, char *m );

GUI_FUNCS gui_funcs[] = {
	{ gui_init },		/**< init */
	{ gui_main },		/**< main */
	{ gui_disconnect },	/**< disconnect */
	{ gui_connected },	/**< conectedo */
	{ gui_habilitado },	/**< habilitado */
	{ gui_start },		/**< start */ 
	{ gui_status },		/**< status */
	{ gui_exit },		/**< exit */
	{ gui_textmsg },	/**< textmsg */
	{ gui_fichas },		/**< fichas */
	{ gui_fichas },		/**< fichas2. Misma que fichas*/
	{ gui_turno },		/**< turno */
	{ gui_ataque },		/**< se esta atacando */
	{ gui_pais },		/**< redibuja un pais */
	{ gui_pais_select },	/**< selecciona un pais */
	{ gui_tropas },		/**< tropas */
	{ gui_tarjeta },	/**< tarjeta */
	{ gui_fichas },		/**< fichasc. Misma que fichas */
	{ gui_dados },		/**< dados. Mostrar dados */
	{ gui_reagrupe },	/**< muestra ventana de reagrupe */
	{ gui_sensi },		/**< muestra sensitiviness en algunos botones */
	{ gui_canje },		/**< Dice que se hizo bien el canje */
	{ gui_lost },		/**< Un jugador perdio */
	{ gui_winner },		/**< Un jugador gano */
	{ gui_objetivo },	/**< Muestra tu objetivo */
	{ gui_textplayermsg },	/**< Mensajes de los jugadores */
};
	
struct _gui_private gui_private;

TTheme	gui_theme;

static TEG_STATUS get_default_values();

static const struct poptOption options[] = {
#ifdef WITH_GGZ
	{"ggz", '\0', POPT_ARG_NONE, &g_juego.with_ggz, 0, N_("Enables GGZ mode"), NULL},
#endif /* WITH_GGZ */
	{"observe", '\0', POPT_ARG_NONE, &g_juego.observer, 0, N_("Observe the game, dont play it"), NULL},
	{NULL, '\0', 0, NULL, 0}
};


/**
 * @fn TEG_STATUS gui_objetivo()
 * Muestra tu objetivo
 */
TEG_STATUS gui_objetivo()
{
	objetivo_view();
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS gui_habilitado( int numjug)
 */
TEG_STATUS gui_habilitado( int numjug )
{
	set_sensitive_tb();
	ministatus_update();
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS gui_winner(int numjug)
 */
TEG_STATUS gui_winner( int numjug, int objetivo )
{
	char buf[400];
	PJUGADOR pJ;

	if( jugador_whois( numjug, &pJ) != TEG_STATUS_SUCCESS) {
		return TEG_STATUS_ERROR;
	}

	if( numjug == WHOAMI() ) {
		snprintf(buf,sizeof(buf)-1,_("Game over, you are the winner!\n\nThe accomplished mission was:\n%s"),_(g_objetivos[objetivo].nombre));
		teg_dialog(_("Game Over"),_("You are the winner"),buf);
	} else {
		snprintf(buf,sizeof(buf)-1,_("Game over, Player %s(%s) is the winner\n\nThe accomplished mission was:\n%s"),pJ->nombre,_(g_colores[pJ->color]), _(g_objetivos[objetivo].nombre));
		teg_dialog(_("Game Over"),_("A player won the game"),buf);
	}
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS gui_lost(int numjug)
 */
TEG_STATUS gui_lost(int numjug)
{
	char buf[200];
	PJUGADOR pJ;

	if( jugador_whois( numjug, &pJ) != TEG_STATUS_SUCCESS) {
		return TEG_STATUS_ERROR;
	}

	if( numjug == WHOAMI() ) {
		snprintf(buf,sizeof(buf)-1,_("You lost the game :("));
		teg_dialog(_("You lost"),_("Game Over"),buf);
	} else {
		snprintf(buf,sizeof(buf)-1,_("Player %s(%s) lost the game"),pJ->nombre,_(g_colores[pJ->color]));
		teg_dialog(_("A player lost the game"),_("Player lost the game"),buf);
	}
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS gui_exit(void)
 */
TEG_STATUS gui_exit(void)
{
	gtk_main_quit();
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS gui_init( int argc, char **argv)
 */
TEG_STATUS gui_init( int argc, char **argv)
{
	GnomeClient *client;

	setlocale(LC_ALL,"");
	bindtextdomain(PACKAGE, GNOMELOCALEDIR);
	textdomain(PACKAGE);

	gnome_init_with_popt_table("TEG", VERSION, argc, argv, options, 0, NULL);

	stock_init();

	client = gnome_master_client();

	get_default_values();

	theme_load(g_juego.theme);

	if( theme_giveme_theme(&gui_theme) != TEG_STATUS_SUCCESS ) {
		fprintf(stderr,"Error loading theme!\n");
		return TEG_STATUS_ERROR;
	}

	main_window = create_mainwin ();

	priv_init();

	if( colors_allocate() != TEG_STATUS_SUCCESS )
		return TEG_STATUS_SUCCESS;

	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS gui_main(void)
 */
TEG_STATUS gui_main(void)
{

	/* show welcome message */
	textmsg(M_ALL,_("Tenes Empanadas Graciela - Gnome client v%s - by Ricardo Quesada"),VERSION);
	textmsg(M_ALL,_("Using theme `%s - v%s' by %s\n"),g_juego.theme, gui_theme.version,gui_theme.author);

	/* poner en sensitive los botones*/
	set_sensitive_tb();

	/* mostrar ventana de coneccion */
	conectar_view();

#ifdef WITH_GGZ
	if( g_juego.with_ggz && g_juego.fd < 0)
		return TEG_STATUS_ERROR;
#endif /* WITH_GGZ */

	gtk_main();
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS gui_disconnect(void)
 */
TEG_STATUS gui_disconnect(void)
{
	if( gui_private.tag >=0 )
		gdk_input_remove( gui_private.tag );
	gui_private.tag=-1;
	set_sensitive_tb();
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS gui_connected( char *c)
 */
TEG_STATUS gui_connected( char *c)
{
	set_sensitive_tb();
	if( !g_juego.observer)
		colortype_view( c );

	return TEG_STATUS_SUCCESS;
}

static TEG_STATUS get_default_values()
{
	gchar temporal[PLAYERNAME_MAX_LEN];
	snprintf(temporal,sizeof(temporal)-1,"/gteg/data/playername=%s",getenv("LOGNAME"));
	strncpy(g_juego.myname,gnome_config_get_string_with_default(temporal,NULL),PLAYERNAME_MAX_LEN);
	g_juego.mycolor = gnome_config_get_int_with_default("/gteg/data/color=1",NULL);

	strncpy(g_juego.sername,gnome_config_get_string_with_default("/gteg/data/servername=localhost",NULL),SERVER_NAMELEN);
	g_juego.serport = gnome_config_get_int_with_default("/gteg/data/port=2000",NULL);
	g_juego.msg_show = gnome_config_get_int_with_default("/gteg/data/msgshow=254",NULL);
	gui_private.msg_show_colors =  gnome_config_get_int_with_default("/gteg/data/msgshow_with_color=0",NULL);
	strncpy(g_juego.theme,gnome_config_get_string_with_default("/gteg/data/theme=modern",NULL),sizeof(g_juego.theme));
	
	gnome_config_set_int   ("/gteg/data/port",  g_juego.serport);
	gnome_config_set_string("/gteg/data/servername",g_juego.sername);
	gnome_config_set_string("/gteg/data/playername",g_juego.myname);
	gnome_config_set_string("/gteg/data/theme",g_juego.theme);
	gnome_config_set_int("/gteg/data/color",g_juego.mycolor);
	gnome_config_set_int("/gteg/data/msgshow",g_juego.msg_show);
	gnome_config_set_int("/gteg/data/msgshow_with_color",gui_private.msg_show_colors);
	gnome_config_sync();
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS gui_start(void)
 * Dice que se empezo a jugar
 */
TEG_STATUS gui_start(void)
{
	cards_flush();
	set_sensitive_tb();
	if( status_dialog!= NULL) out_status();
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS gui_status()
 * Hace update del 'players status window'
 */
TEG_STATUS gui_status()
{
	status_update();
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS gui_fichas(int cant, int conts)
 * Avisa la cantidad de ejercitos que hay que poner
 * @param cant cantidad de ejercitos
 * @conts fichas que hay que poner por continentes conquistados
 */
TEG_STATUS gui_fichas(int cant, int conts)
{
	char buf[500];
	JUG_ESTADO e;

	set_sensitive_tb();

	if( status_dialog != NULL) out_status();

	e = ESTADO_GET();

	armies_view( cant, conts );

	switch(e) {
	case JUG_ESTADO_FICHAS:
		sprintf(buf,_("1st part:\nPlace %d armies in your countries"),cant);
		break;
	case JUG_ESTADO_FICHAS2:
		sprintf(buf,_("2nd part:\nPlace %d armies in your countries"),cant);
		break;
	case JUG_ESTADO_FICHASC:
		{
			int i,p1,p2,p3;
			char buf2[500];
			int hubo_cont=0;

			buf[0]=0;
			for(i=0;i<CONT_CANT;i++) {
				if( conts & 1 ) {
					sprintf(buf2,_("Place %d armies in %s\n"),g_conts[i].fichas_x_cont,_(g_conts[i].nombre));
					strcat(buf,buf2);
					hubo_cont=1;
				}
				conts >>= 1;
			}
			if( hubo_cont ) {
				sprintf(buf2,_("And place another %d armies in your countries\n"),cant);
				strcat(buf,buf2);
			} else {
				sprintf(buf2,_("Place %d armies in your countries\n"),cant);
				strcat(buf,buf2);
			}

			if( canje_puedo(&p1,&p2,&p3) == TEG_STATUS_SUCCESS ) {
				strcat(buf,_("\nIf you want to EXCHANGE your cards for armies,\nDO IT NOW!"));
				cards_select(p1,p2,p3);
			}
			break;
		}
	default:
		textmsg(M_ERR,_("Error in out_fichas()"));
		return TEG_STATUS_ERROR;
	}

	teg_dialog(_("Place armies"),_("Place armies"),buf);
	textmsg(M_IMP,"%s",buf);

	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS gui_turno()
 * Avisa que es mi turno de atacar
 */
TEG_STATUS gui_turno()
{
	set_sensitive_tb();

	if( status_dialog != NULL) out_status();

	teg_dialog(_("Its your turn"),_("Its your turn to attack"),
		_("Select two countries:\n1st: click on the source country\n2nd: click on the destination country"));

	return TEG_STATUS_SUCCESS;
}


/**
 * @fn TEG_STATUS gui_pais( int p )
 * muestra un pais
 * @param p Pais a mostrar
 */
TEG_STATUS gui_pais( int p )
{
//	if( status_dialog != NULL) out_status();

	if( p < PAISES_CANT ) {
		G_pais_draw_ejer( p );
	}

	return TEG_STATUS_SUCCESS;
}


/**
 * @fn TEG_STATUS gui_tropas( int src, int dst, int cant )
 * Dice que se conquisto un pais y le permite pasar ejercitos de un pais a otro
 * @param src Pais origen
 * @param dst Pais destino
 * @param cant Cantidad de ejercitos como max que se pueden pasar
 */
TEG_STATUS gui_tropas( int src, int dst, int cant )
{
	tropas_window( src, dst, cant );
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS gui_reagrupe( int src, int dst, int cant )
 * Dice las fichas que se van a pasar de un pais a otro
 * @param src Pais origen
 * @param dst Pais destino
 * @param cant Cantidad de ejercitos como max que se pueden pasar
 */
TEG_STATUS gui_reagrupe( int src, int dst, int cant )
{
	reagrupe_window( src, dst, cant );
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS gui_tarjeta( int pais, int usado )
 * Llamado cuando recibo una tarjeta
 * @param pais Tarjeta del pais que recibo
 */
TEG_STATUS gui_tarjeta( int pais )
{
	cards_view( pais );
	set_sensitive_tb();
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS gui_dados()
 * Llamado cuando recibo una tarjeta
 * @param pais Tarjeta del pais que recibo
 * @para usado Si la tarjeta la uso el server automaticamente (ver token_tarjeta en server)
 */
TEG_STATUS gui_dados()
{
	dices_view();
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS gui_sensi()
 * Para poner sentitiviness en algunos botones
 */
TEG_STATUS gui_sensi()
{
	set_sensitive_tb();
	cards_update();
	cards_update_para_canje();
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS gui_canje()
 * Informa que se hizo bien el canje y borra las tarjetas si estan vistas
 * @param cant Cantidad de ejercitos que se pueden agregar por el canje hecho
 * @param p1 Pais 1 del canje hecho
 * @param p2 Pais 2 del canje hecho
 * @param p3 Pais 3 del canje hecho
 */
TEG_STATUS gui_canje( int cant, int p1, int p2, int p3)
{
	char buf[100];
	sprintf(buf,_("Now you can add %d extra armies in your countries!"), cant);
	teg_dialog(_("Extra armies"),_("Extra armies"),buf);
	armies_view_more( cant );
	cards_delete(p1,p2,p3);
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS gui_ataque( int src, int dst )
 * Muestra que se esta atacando de un pais a otro
 * @param src pais que ataca
 * @param dst pais que se defiende
 */
TEG_STATUS gui_ataque( int src, int dst )
{
	gui_pais_select( src );
	gui_pais_select( dst );
	return TEG_STATUS_SUCCESS;
}
