/*		$Id: themes.c,v 1.9 2001/07/26 03:58:56 riq Exp $	*/
/* Tenes Empanadas Graciela
 *
 * Copyright (C) 2001 Ricardo Quesada
 *
 * Author: Ricardo Calixto Quesada <rquesada@core-sdi.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
/**
 * @file themes.c
 * File that parse the theme stuff
 */

/*
 * File based in the job example from libxml
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <dirent.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#include "all.h"
#include "common.h"
#include "themes.h"

#ifndef xmlChildrenNode
#define xmlChildrenNode childs
#define xmlRootNode root
#endif


static pTheme g_theme = NULL;	/**< Current theme */
static pTInfo g_tinfo = NULL;	/**< info of all current themes */

static pCountry
parseCountry(xmlDocPtr doc, xmlNodePtr cur)
{
	pCountry ret = NULL;

	ret = (pCountry) malloc(sizeof(Country));

	if (ret == NULL) {
		fprintf(stderr,_("Out of memory\n"));
		return(NULL);
	}
	
	memset(ret, 0, sizeof(*ret));
	
	ret->name = xmlGetProp(cur, (const xmlChar *) "name");
	if (ret->name == NULL)
		fprintf(stderr, _("Country has no name\n"));

	ret->file = xmlGetProp(cur, (const xmlChar *) "file");
	if (ret->file == NULL)
		fprintf(stderr, _("Country has no file\n"));

	ret->pos_x = xmlGetProp(cur, (const xmlChar *) "pos_x");
	if (ret->pos_x == NULL)
		fprintf(stderr, _("Country has no pos_x\n"));

	ret->pos_y = xmlGetProp(cur, (const xmlChar *) "pos_y");
	if (ret->pos_y == NULL)
		fprintf(stderr, _("Country has no pos_y\n"));

	ret->army_x = xmlGetProp(cur, (const xmlChar *) "army_x");
	if (ret->army_x == NULL)
		fprintf(stderr, _("Country has no army_x\n"));

	ret->army_y = xmlGetProp(cur, (const xmlChar *) "army_y");
	if (ret->army_y == NULL)
		fprintf(stderr, _("Country has no army_y\n"));

	cur = cur->xmlChildrenNode;

	if ( cur != NULL ) {
		fprintf(stderr,_("Wrong type (%s). Nothing was expected\n"), cur->name);
		free(ret);
		return(NULL);
	}

	return(ret);
}

static pContinent
parseContinent(xmlDocPtr doc, xmlNodePtr cur)
{
	pContinent ret = NULL;
	pCountry curcountry = NULL;

	ret = (pContinent) malloc(sizeof(Continent));

	if (ret == NULL) {
		fprintf(stderr,_("Out of memory\n"));
		return(NULL);
	}
	
	memset(ret, 0, sizeof(*ret));
	
	ret->name = xmlGetProp(cur, (const xmlChar *) "name");
	if (ret->name == NULL)
		fprintf(stderr, _("Continent has no name\n"));

	ret->pos_x = xmlGetProp(cur, (const xmlChar *) "pos_x");
	if (ret->pos_x == NULL)
		fprintf(stderr, _("Continent has no pos_x\n"));

	ret->pos_y = xmlGetProp(cur, (const xmlChar *) "pos_y");
	if (ret->pos_y == NULL)
		fprintf(stderr, _("Continent has no pos_y\n"));

	cur = cur->xmlChildrenNode;
	while( cur != NULL ) {
		/* country */
		if ( !xmlStrcmp(cur->name, (const xmlChar *) "country") ) {
			curcountry = parseCountry( doc, cur );
			if (curcountry != NULL) {
				ret->countries[ret->i_country++] = curcountry;
				if (ret->i_country >= THEME_COUNTRY_MAX)
					break;
			}
		}

		/* else */
		else {
			fprintf(stderr,_("Wrong type (%s). country was expected\n"), cur->name);
			free(ret);
			return(NULL);
		}

		cur = cur->next;
	}

	return(ret);
}

static pTheme parseTheme(char *filename)
{
	xmlDocPtr doc;
	pContinent curcontinent = NULL;
	xmlNodePtr cur;
	pTheme ret = NULL;

	/*
	 * build an XML tree from a the file;
	 */
	doc = xmlParseFile(filename);
	if (doc == NULL) return(NULL);

	/*
	 * Check the document is of the right kind
	 */
	
	cur = xmlDocGetRootElement(doc);
	if (cur == NULL) {
		fprintf(stderr,_("Empty document\n"));
		xmlFreeDoc(doc);
		return(NULL);
	}

	if (xmlStrcmp(cur->name, (const xmlChar *) "teg_theme")) {
		fprintf(stderr,_("Wrong type. root node != teg_theme\n"));
		xmlFreeDoc(doc);
		return(NULL);
	}

	/*
	 * Allocate the structure to be returned.
	 */
	ret = (pTheme) malloc(sizeof(Theme));
	if (ret == NULL) {
		fprintf(stderr,_("Out of memory\n"));
		xmlFreeDoc(doc);
		return(NULL);
	}
	memset(ret, 0, sizeof(*ret));

	/*
	 * Now, walk the tree.
	 */
	cur = cur->xmlChildrenNode;
#if 0
	while ( cur && xmlIsBlankNode ( cur ) ) {
		cur = cur -> next;
	}
#endif
	if ( cur == 0 ) {
		free( ret );
		return ( NULL );
	}

	while (cur != NULL) {

		/* author */
		if (!xmlStrcmp(cur->name, (const xmlChar *) "author") ) {
			ret->author = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			if (ret->author== NULL)
				fprintf(stderr, _("Theme:Author is empty\n"));
		}

		else if (!xmlStrcmp(cur->name, (const xmlChar *) "version") ) {
			ret->version= xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			if (ret->version == NULL)
				fprintf(stderr, _("Theme:version is empty\n"));
		}


		/* continents */
		else if (!xmlStrcmp(cur->name, (const xmlChar *) "continent") ) {
			curcontinent = parseContinent( doc, cur );
			if (curcontinent != NULL) {
				ret->continents[ret->i_continent++] = curcontinent;
				if (ret->i_continent>= THEME_CONTINENT_MAX)
					break;
			}
		}

		/* board */
		else if (!xmlStrcmp(cur->name, (const xmlChar *) "board") ) {
			ret->board = xmlGetProp(cur, (const xmlChar *) "file");
			if (ret->board == NULL)
				fprintf(stderr, _("Board has no name\n"));
		}

		/* dices pos */
		else if (!xmlStrcmp(cur->name, (const xmlChar *) "dices_pos") ) {

			ret->dices_x = xmlGetProp(cur, (const xmlChar *) "x");
			if (ret->dices_x== NULL)
				fprintf(stderr, _("Dices_pos has no x\n"));
			ret->dices_y = xmlGetProp(cur, (const xmlChar *) "y");
			if (ret->dices_y== NULL)
				fprintf(stderr, _("Dices_pos has no y\n"));
		}

		/* dices text */
		else if (!xmlStrcmp(cur->name, (const xmlChar *) "dices_text") ) {

			ret->dices_color = xmlGetProp(cur, (const xmlChar *) "color");
			if (ret->dices_color== NULL)
				fprintf(stderr, _("Dices_text has no color\n"));
		}

		/* screenshot */
		else if (!xmlStrcmp(cur->name, (const xmlChar *) "screenshot") ) {
			ret->screenshot = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			if (ret->author== NULL)
				fprintf(stderr, _("Theme:Screenshot is empty\n"));
		}

#ifdef XML_WORKS_OK
		/* dices file */
		else  if (!xmlStrcmp(cur->name, (const xmlChar *) "dices_file") ) {
			int i;

			for(i=0;i<DICES_CANT;i++) {
				char name[40];

				memset(name,0,sizeof(name));
				snprintf(name,sizeof(name)-1,"%d",i+1);
				ret->dices_file[i] = xmlGetProp(cur, (const xmlChar *) name);
				if (ret->dices_file[i]== NULL)
					fprintf(stderr, _("Dices_file has no %d\n"),i);
			}
		}
#endif	/* XML_WORKS_OK */
		else {
			fprintf(stderr,_("Wrong type (%s). continent, board and dices was expected\n"), cur->name);
			xmlFreeDoc(doc);
			free(ret);
			return(NULL);
		}

		cur = cur->next;
	}

	return(ret);
}

static TEG_STATUS theme_fill_country_name()
{
	int j,k,l;

	if( !g_theme )
		return TEG_STATUS_ERROR;

	l=0;
	for(j=0;j<g_theme->i_continent;j++) {
		for(k=0;k<g_theme->continents[j]->i_country;k++) {
			g_paises[l++].nombre = g_theme->continents[j]->countries[k]->name;
		}
	}

	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS theme_load( char *name)
 * Loads the 'name' theme
 */
TEG_STATUS theme_load( char *name)
{
	char filename[512];

	memset(filename,0,sizeof(filename));

	strncat( filename,"themes/", sizeof(filename)-1);
	strncat( filename, name, sizeof(filename)-1);
	strncat( filename, "/teg_theme.xml", sizeof(filename)-1);

	if( g_theme )
		theme_unload();

	if( (g_theme = parseTheme( filename )) == NULL) {
		memset(filename,0,sizeof(filename));
		strncpy(filename,THEMEDIR,sizeof(filename)-1);
		strncat(filename,name,sizeof(filename)-1);
		strncat( filename, "/teg_theme.xml", sizeof(filename)-1);
		if( (g_theme = parseTheme( filename )) == NULL) {
			fprintf(stderr, _("Unable to load theme `%s'\n"),name);
			return TEG_STATUS_THEMEERROR;
		}
	}

	/* place new name of the countries */
	theme_fill_country_name();

	return TEG_STATUS_SUCCESS;
}

void theme_unload()
{
}

/**
 * @fn TEG_STATUS theme_giveme_continent(pTContinent pC, int n)
 */
TEG_STATUS theme_giveme_continent(pTContinent pC, int n)
{
	if( g_theme == NULL )
		return TEG_STATUS_ERROR;

	if( n <0 || n >= g_theme->i_continent )
		return TEG_STATUS_ERROR;

	pC->pos_x = atoi( g_theme->continents[n]->pos_x );
	pC->pos_y = atoi( g_theme->continents[n]->pos_y );

	return TEG_STATUS_SUCCESS;
}
/**
 * @fn TEG_STATUS theme_giveme_country(pTCountry pC, int n)
 */
TEG_STATUS theme_giveme_country(pTCountry pC, int cont, int n)
{
	if( g_theme == NULL )
		return TEG_STATUS_ERROR;

	if( cont <0 || cont >= g_theme->i_continent )
		return TEG_STATUS_ERROR;

	if( n <0 || n >= g_theme->continents[cont]->i_country )
		return TEG_STATUS_ERROR;

	pC->name = g_theme->continents[cont]->countries[n]->name;
	pC->file = g_theme->continents[cont]->countries[n]->file;
	pC->pos_x= atoi( g_theme->continents[cont]->countries[n]->pos_x);
	pC->pos_y= atoi( g_theme->continents[cont]->countries[n]->pos_y);
	pC->army_x = atoi( g_theme->continents[cont]->countries[n]->army_x);
	pC->army_y = atoi( g_theme->continents[cont]->countries[n]->army_y);

	return TEG_STATUS_SUCCESS;
}

TEG_STATUS theme_giveme_theme(pTTheme pT)
{
	int i;

	if( g_theme == NULL )
		return TEG_STATUS_ERROR;

	pT->author= g_theme->author;
	pT->version = g_theme->version;
	pT->board = g_theme->board;
	pT->dices_x= atoi( g_theme->dices_x );
	pT->dices_y= atoi( g_theme->dices_y );

	for(i=0;i<DICES_CANT;i++)
		pT->dices_file[i] = g_theme->dices_file[i];

	pT->dices_color = g_theme->dices_color;
	pT->screenshot = g_theme->screenshot;

	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS theme_enum_themes( pTInfo pTI )
 * builds a list of available themes
 */
TEG_STATUS theme_enum_themes( pTInfo pTI )
{
	char *dname = "themes";
	DIR *dir;
//	DIR *dir_xml;
//	struct dirent *e;
	int i;
	char *themes[] = { "classic", "modern", NULL };
	pTInfo pI;


	if( g_tinfo ) {
		*pTI = *g_tinfo;
		return TEG_STATUS_SUCCESS;
	}

	if( (dir = opendir (dname))==NULL) {
		if( (dir = opendir( THEMEDIR ))==NULL) {
			fprintf(stderr,_("Themes not installed"));
			return TEG_STATUS_ERROR;
		}
		closedir (dir);
	}

	/* XXX why is this not fucking working ???!!! */
#if 0
	while ((e = readdir (dir)) != NULL) {
		if(( (dir_xml = opendir(e->d_name)) != NULL ) && e->d_name[0]!='.' ) {
			char *s = strdup (e->d_name);
			closedir(dir_xml);
		}
	}
#endif

	for(i=0; themes[i] != NULL; i++ ) {
		pI = malloc( sizeof(*pI));
		if( pI == NULL )
			return TEG_STATUS_NOMEM;

		memset(pI,0,sizeof(*pI));
		pI->name = strdup(themes[i]);

		/* insert it in the list */
		if( g_tinfo == NULL ) {
			pI->next = NULL;
			g_tinfo = pI;
		} else {
			pI->next = g_tinfo->next;
			g_tinfo->next = pI;
		}
	}

	*pTI = *g_tinfo;
	return TEG_STATUS_SUCCESS;
}

TEG_STATUS theme_init()
{
	g_tinfo = NULL;
	return TEG_STATUS_SUCCESS;
}

void theme_free()
{
	pTInfo pI;

	for( pI = g_tinfo; pI != NULL; )  {
		pTInfo pI2;
		if(pI->name)
			free( pI->name );
		pI2 = pI;
		pI = pI->next;
		free(pI2);
	}
}
