/*	$Id: ai.c,v 1.21 2001/04/30 03:53:36 riq Exp $	*/
/* Tenes Empanadas Graciela
 *
 * Copyright (C) 2000 Ricardo Quesada
 *
 * Author: Ricardo Calixto Quesada <riq@core-sdi.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
/**
 * @file ai.c
 * Inteligecia artificial del robot
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <unistd.h>

#include "client.h"
#include "ai.h"

static int puntaje[PAISES_CANT];
static int sorted[PAISES_CANT];
static int attack_dst[PAISES_CANT];


/**
 * @fn TEG_STATUS ai_puntaje_sort( int cant )
 * Pone los 'cant' puntajes mas altos en sorted.
 * Puede que uno este varias veces.
 */
TEG_STATUS ai_puntaje_sort( int cant)
{
	int i,j;
	int max;

	for(j=0;j<cant;j++) {
		max = 0;
		for(i=0;i<PAISES_CANT;i++) {
			if( puntaje[i] > puntaje[max] )
				max = i;
		}
		sorted[j] = max;
		puntaje[max] -= 2;
	}
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS ai_puntaje_sort_for_attack( int cant)
 * Algoritmo para ordernar como se va a hacer el ataque
 */
TEG_STATUS ai_puntaje_sort_for_attack()
{
	int i;
	int max=0;

	for(i=0;i<PAISES_CANT;i++) {
		if( puntaje[i] > puntaje[max] )
			max = i;
	}
	sorted[0] = max;
	return TEG_STATUS_SUCCESS;
}


/**
 * @fn TEG_STATUS ai_puntaje_clean()
 */
TEG_STATUS ai_puntaje_clean()
{
	int i;
	for(i=0;i<PAISES_CANT;i++)
		puntaje[i] = -10000;

	return TEG_STATUS_SUCCESS;
}

/**
 * @fn int ai_many_pais_enemigo( int p )
 * Dice la cantidad de paises enemigos que tiene un pais
 */
int ai_many_pais_enemigo( int p )
{
	int i;
	int c=0;

	for(i=0;i<PAISES_CANT;i++) {
		if( g_paises[i].numjug != WHOAMI() && paises_eslimitrofe(p,i))
			c++;
	}

	return c;
}

/**
 * @fn BOOLEAN ai_is_frontier( int p )
 * Dice si un pais es frontera con otro continete
 */
BOOLEAN ai_is_frontier( int p )
{
	int i;
	for(i=0;i<PAISES_CANT;i++) {
		if( paises_eslimitrofe(i,p) && g_paises[i].continente != g_paises[p].continente )
			return TRUE;
	}

	return FALSE;
}

/**
 * @fn BOOLEAN ai_own_contienent( int c )
 * Dice si soy owner de un continente
 */
BOOLEAN ai_own_continent( int c )
{
	int i;
	int t=0;

	for(i=0;i<PAISES_CANT;i++) {
		if( g_paises[i].numjug == WHOAMI() && g_paises[i].continente == c )
			t++;
	}

	return ( t == g_conts[c].cant_paises );

}

/**
 * @fn TEG_STATUS ai_fichas_calc_puntaje_conquer( int pais )
 * Calcula puntaje segun convenga conquistar el continente
 */
TEG_STATUS ai_fichas_calc_puntaje_conquer( int pais )
{
	int i;
	int pc;			/* paises del continente */
	int pe=0;		/* paises enemigos */
	int ee=0;		/* ejercitos enemigos */
	int pm=0;		/* paises mios */
	int em=0;		/* ejercitos mios */
	int ple=0;		/* paises limitrofes enemigos */

	pc = g_conts[ g_paises[pais].continente ].cant_paises;

	for(i=0;i<PAISES_CANT;i++) {
		if( g_paises[pais].continente == g_paises[i].continente ) {
			if( g_paises[i].numjug == WHOAMI() ) {
				pm++;
				em += g_paises[i].ejercitos;
			} else {
				pe++;
				ee += g_paises[i].ejercitos;
				if( paises_eslimitrofe(pais,i)) {
					ple++;
				}
			}
		}
	}

	/* si tengo mas del xx% de los paises del cont suma puntos */
	i = pm*100/pc;
	if( i < 99 ) {
		if( i >= 50 )
			puntaje[pais] += 3;
		if( i >= 65 )
			puntaje[pais] += 4;
		if( i >= 80 )
			puntaje[pais] += 5;
		puntaje[pais] += ple;
	}

	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS ai_fichas_calc_puntaje_fichas( int pais )
 * Calcula puntaje segun los paises limitrofes las fichas enemigas que lo rodeen
 */
TEG_STATUS ai_fichas_calc_puntaje_fichas( int pais )
{
	int i;
	int p1=0;
	int p2=0;

	/* suma las fichas de los paises limitrofes enemigos */
	for(i=0;i< PAISES_CANT;i++) {
		if( g_paises[i].numjug != WHOAMI() && paises_eslimitrofe( pais, i )) {

			/* si es un enemigo ya tiene punto */
			puntaje[ pais ] += 3;

			if( g_paises[pais].ejercitos >= g_paises[i].ejercitos )
				p2 += 2 + g_paises[pais].ejercitos - g_paises[i].ejercitos;
			else
				p1 +=g_paises[i].ejercitos - g_paises[pais].ejercitos;
		}
	}

	puntaje[ pais ] += p1/2 + p2/2;

	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS ai_fichas_calc_puntaje_cont( int pais )
 * Calcula puntaje segun los paises limitrofes las fichas enemigas que lo rodeen
 */
TEG_STATUS ai_fichas_calc_puntaje_cont( int pais )
{
	int i;
	int p;			/* puntaje temp */

	p = 0;

	/* suma las fichas de los paises limitrofes enemigos */
	for(i=0;i< PAISES_CANT;i++) {

		/* todos los paises del continente tienen puntos */
		if( g_paises[pais].continente == g_paises[i].continente )
			p += 3;

		/* los limitrofes con otros continente tienen mas */
		if( paises_eslimitrofe( pais, i ) && g_paises[pais].continente != g_paises[i].continente) {
			p += 7;

			/* y si ademas es limitrofe con un enemigo tienen 7 puntos mas */
			if( g_paises[i].numjug != WHOAMI() )
				p += 11;
		}
	}

	puntaje[ pais ] += p;
	return TEG_STATUS_SUCCESS;
}


/**
 * @fn TEG_STATUS __ai_fichas( int cant )
 * Pone fichas en los paises.
 */
TEG_STATUS __ai_fichas( int cant )
{
	int i;

	ai_puntaje_clean();
	for(i=0;i< PAISES_CANT;i++ ) {
		if( g_paises[i].numjug == WHOAMI() ) {
			puntaje[i] = 0;
			ai_fichas_calc_puntaje_conquer( i );
			ai_fichas_calc_puntaje_fichas( i );
		}
	}
	ai_puntaje_sort( cant );
	for(i=0;i<cant;i++) {
		if( fichas_add( &g_paises[ sorted[i] ]) != TEG_STATUS_SUCCESS ) {
			return TEG_STATUS_ERROR;
		}
	}
	return TEG_STATUS_SUCCESS;
}
/**
 * @fn TEG_STATUS ai_fichas( int cant )
 */
TEG_STATUS ai_fichas( int cant )
{
	if( __ai_fichas( cant ) != TEG_STATUS_SUCCESS )
		return TEG_STATUS_ERROR;

	if( fichas_out() != TEG_STATUS_SUCCESS )
		return TEG_STATUS_ERROR;

	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS ai_fichas_en_cont( int cont )
 * Pone las fichas para el continente
 */
TEG_STATUS ai_fichas_en_cont( int cont )
{
	int cant = g_conts[cont].fichas_x_cont;
	int i;

	ai_puntaje_clean();
	for(i=0;i<PAISES_CANT;i++) {
		if( g_paises[i].continente == g_conts[cont].id ) {
			ai_fichas_calc_puntaje_cont( i );
		}
	}

	ai_puntaje_sort( cant );
	for(i=0;i<cant;i++) {
		if( fichas_add( &g_paises[ sorted[i] ]) != TEG_STATUS_SUCCESS ) {
			return TEG_STATUS_ERROR;
		}
	}

	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS ai_fichasc( int cant, int conts)
 * pone fichas de continente y de eso.
 */
TEG_STATUS ai_fichasc( int cant, int conts)
{
	int i;
	int conts_tmp;

	/* pongo las fichas del continente */
	conts_tmp = conts;
	for(i=0;i<CONT_CANT;i++) {
		if( conts_tmp & 1 ) {
			ai_fichas_en_cont( i );
		}
		conts_tmp >>= 1;
	}

	/* y ahora las fichas donde creo conveniente */
	if( __ai_fichas( cant ) != TEG_STATUS_SUCCESS )
		return TEG_STATUS_ERROR;

	if( fichas_out() != TEG_STATUS_SUCCESS )
		return TEG_STATUS_ERROR;

	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS ai_init()
 */
TEG_STATUS ai_init()
{
	ai_puntaje_clean();
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn BOOLEAN ai_is_pais_peligroso( int src, int dst )
 * Dice si un pais el peligroso, dependiendo si este lo puede
 * atacar antes de que se acabe el turno
 */
BOOLEAN ai_is_pais_peligroso( int src, int dst )
{
	int aparecio_empezo = FALSE;
	int dst_jugo = FALSE;
	PLIST_ENTRY l = g_list_jugador.Flink;
	PJUGADOR pJ;
	int tmp;

	while( !IsListEmpty( &g_list_jugador ) && (l != &g_list_jugador) ) {
		pJ = (PJUGADOR) l;

		if( pJ->empezo_turno )
			aparecio_empezo = TRUE;

		if( pJ->numjug == g_paises[dst].numjug ) {
			if( aparecio_empezo )
				dst_jugo = TRUE;
			else
				dst_jugo = FALSE;
			break;
		}

		l = LIST_NEXT(l);
	}

	if( dst_jugo )
		return FALSE;

	tmp = (g_paises[src].ejercitos > 3) ? 3: g_paises[src].ejercitos-1;
	if( g_paises[dst].ejercitos > (g_paises[src].ejercitos-tmp) )
		return TRUE;
	return FALSE;
}

/**
 * @fn int ai_puntaje_atacar_warning( int src )
 * Resta puntos si existe un pais limitrofe a 'src' que puede atacarlo antes
 * de que se acabe el turno
 */
int ai_puntaje_atacar_warning( int src )
{
	int p=0;
	int i;

	for(i=0;i<PAISES_CANT;i++) {
		if( paises_eslimitrofe(src,i) &&
			g_paises[src].numjug != g_paises[i].numjug &&
			ai_is_pais_peligroso(src,i)) {

			p += (g_paises[i].ejercitos - g_paises[src].ejercitos);
		}
	}

	return -(5+2*p);
}


/**
 * @fn int ai_puntaje_atacar_pais( int src, int dst )
 * Dice los puntos que tiene de atacar src a dst
 */
int ai_puntaje_atacar_pais( int src, int dst )
{
	int p=0;

	if( g_paises[src].ejercitos > g_paises[dst].ejercitos ) {

		if( g_paises[src].continente == g_paises[dst].continente )
			p++;
		
		p += 10 + g_paises[src].ejercitos - g_paises[dst].ejercitos;

	} else {

		int r = RANDOM_MAX(1,10);

		if((r>7) && ( g_paises[src].ejercitos > 7 ))
			p++;
	}

	return p;
}

/**
 * @fn int ai_puntaje_ataque( int p )
 * Evalua los puntos que tiene el pais para atacar
 * y dice al pais que convendria atacar
 */
TEG_STATUS ai_puntaje_atacar( int pais )
{
	int i;
	int p=0;
	int p_tmp;
	int dst=-1;

	for(i=0;i<PAISES_CANT;i++) {
		if( paises_eslimitrofe(pais,i) && g_paises[i].numjug !=WHOAMI() ) {
			p_tmp = ai_puntaje_atacar_pais( pais, i );

			/* son negativos estos valores, por eso sumo */
			p_tmp += ai_puntaje_atacar_warning(pais);

			if( p_tmp > p ) {
				p = p_tmp;
				dst = i;
			}
		}
	}

	/* return the best candidate */
	if( dst >=0 ) {
		puntaje[ pais ] = p;
		attack_dst[ pais ] = dst;
	}

	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS ai_turno()
 * Hace los ataques correspondientes
 */
TEG_STATUS ai_turno()
{
	int i;
	int j;

	ai_puntaje_clean();
	for(i=0;i<PAISES_CANT;i++) {
		if( g_paises[i].numjug == WHOAMI() && g_paises[i].ejercitos > 1)
			ai_puntaje_atacar(i);
	}
	ai_puntaje_sort_for_attack();

	i = sorted[0];
	j = attack_dst[i];
	if( puntaje[i] > 0 ) {
		
		ataque_init();
		ataque_click( &g_paises[i]);
		ataque_click( &g_paises[j]);
		return TEG_STATUS_SUCCESS;
	}
	return TEG_STATUS_ERROR;
}

TEG_STATUS __ai_reagrupe( int  p, int cant )
{
	int i;
	int new_cant = cant;	/* para cuando se mueve a otro cont */


	/* si voy a mover paises a otro continente, dejo al menos 2 */
	if( g_paises[p].ejercitos - g_paises[p].ejer_reagrupe - cant < 2 )
		new_cant--;


	/* trato de enviar los ejercitos a otro continente... */
	for(i=0;i<PAISES_CANT && new_cant;i++) {
		if( g_paises[i].numjug == WHOAMI() && paises_eslimitrofe(p,i) &&
				g_paises[i].continente != g_paises[p].continente ) {

			reagrupe_out( p, i, new_cant );
			return TEG_STATUS_SUCCESS;
		}
	}

	/*... si falla, los muevo a una frontera */
	for(i=0;i<PAISES_CANT;i++) {
		if( g_paises[i].numjug == WHOAMI() && paises_eslimitrofe(p,i) &&
				g_paises[i].continente == g_paises[p].continente ) {

			if( ai_is_frontier(i) ) {
				reagrupe_out( p, i, cant );
				return TEG_STATUS_SUCCESS;
			}
		}
	}

	/*... y si no hay frontera lo mano al 1ro que tenga a mano */
	for(i=0;i<PAISES_CANT;i++) {
		if( g_paises[i].numjug == WHOAMI() && paises_eslimitrofe(p,i) &&
				g_paises[i].continente == g_paises[p].continente ) {

			reagrupe_out( p, i, cant );
			return TEG_STATUS_SUCCESS;
		}
	}

	return TEG_STATUS_ERROR;
}

/**
 * @fn TEG_STATUS ai_reagrupe()
 */
TEG_STATUS ai_reagrupe()
{
	int i,p;
	reagrupe_init();

	for(i=0;i < PAISES_CANT ; i++ ) { 
		if( g_paises[i].numjug == WHOAMI() ) {

			p = g_paises[i].ejercitos - g_paises[i].ejer_reagrupe;
			if( p < 2 ) continue;

			if( ai_own_continent( g_paises[i].continente ) && ai_many_pais_enemigo(i) == 0 ) {
				__ai_reagrupe(i,p-1);
			}
		}
	}
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS ai_tropas(int src, int dst, int cant)
 */
TEG_STATUS ai_tropas(int src, int dst, int cant)
{
	int i;
	int c=0;

	ai_puntaje_clean();
	ai_fichas_calc_puntaje_fichas( src );
	ai_fichas_calc_puntaje_fichas( dst );

	for(i=0;i<cant;i++) {
		if( puntaje[dst] > puntaje[src] + i )
			c++;
	}

	if( c > 0 )
		out_tropas( src, dst, c );

	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS ai_puedocanje( int *p1, int *p2, int *p3 )
 * Dice si se puede hacer canje, y con que tarjetas si se puede
 */
TEG_STATUS ai_puedocanje( int *p1, int *p2, int *p3 )
{
	return canje_puedo( p1,p2,p3);
}
