/*	$Id: jugador.c,v 1.40 2001/06/02 22:17:21 riq Exp $	*/
/* Tenes Empanadas Graciela
 *
 * Copyright (C) 2000 Ricardo Quesada
 *
 * Author: Ricardo Calixto Quesada <rquesada@core-sdi.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
/**
 * @file jugador.c
 * Funciones que manejan a los jugadores
 */
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include <unistd.h>

#include "server.h"

LIST_ENTRY g_list_jugador;		/**< puntero a la lista de jugadores */

/**
 * @fn TEG_STATUS jugador_whois( int numjug, PJUGADOR *pJ)
 * Dado un numjug, devuelve el ptr al jugador
 */
TEG_STATUS jugador_whois( int numjug, PJUGADOR *pJ)
{
	PLIST_ENTRY l = g_list_jugador.Flink;
	PJUGADOR pJ_new;

	while( !IsListEmpty( &g_list_jugador ) && (l != &g_list_jugador) ) {
		pJ_new = (PJUGADOR) l;
		if( pJ_new->numjug == numjug) {
			*pJ = pJ_new;
			return TEG_STATUS_SUCCESS;
		}
		l = LIST_NEXT(l);
	}
	return TEG_STATUS_PLAYERNOTFOUND;
}

/**
 * @fn void jugador_initplayer( PJUGADOR pJ )
 * Initialize the player.
 */
TEG_STATUS jugador_initplayer( PJUGADOR pJ )
{
	assert( pJ );

	InitializeListHead( &pJ->paises );
	InitializeListHead( &pJ->pactos );
	InitializeListHead( &pJ->tarjetas );
	pJ->hizo_canje = FALSE;
	pJ->tot_canjes = 0;
	pJ->tot_paises = 0;
	pJ->tot_tarjetas = 0;
	pJ->tot_ejercitos = 0;
	pJ->turno_conq = 0;
	pJ->pais_src = -1;
	pJ->pais_dst = -1;
	pJ->objetivo = -1;

	return TEG_STATUS_SUCCESS;
}

/**
 * @fn void jugador_resetconn( PJUGADOR pJ )
 * Resetea un jugador
 */
void jugador_resetconn( PJUGADOR pJ )
{
	assert( pJ );
	jugador_initplayer(pJ);
	pJ->numjug = -1;
	pJ->fd=-1;
	pJ->estado = JUG_ESTADO_DESCONECTADO;
	pJ->color = -1;
}

/**
 * @fn void jugador_init( void )
 * Inicializador global de los jugadores
 */
void jugador_init( void )
{
	InitializeListHead( &g_list_jugador );
}

/**
 * @fn TEG_STATUS jugador_numjug_libre( int *libre)
 * Devuelve un numjug libre
 * @param puntero para devolver el jug libre
 */
TEG_STATUS jugador_numjug_libre( int *libre)
{
	char jugs[TEG_MAX_PLAYERS];
	int i;
	PLIST_ENTRY l = g_list_jugador.Flink;
	PJUGADOR pJ;

	assert( libre );

	memset(jugs,0,sizeof(jugs));

	while( !IsListEmpty( &g_list_jugador ) && (l != &g_list_jugador) ) {
		pJ = (PJUGADOR) l;
		if( pJ->jugador ) {
			if( pJ->numjug >= 0 && pJ->numjug < TEG_MAX_PLAYERS ) {
				jugs[pJ->numjug] = 1;
			}
		}

		l = LIST_NEXT(l);
	}

	for(i=0;i<TEG_MAX_PLAYERS;i++) {
		if( jugs[i] == 0) {
			*libre = i;
			return TEG_STATUS_SUCCESS;
		}
	}

	/* server is full */
	return TEG_STATUS_ERROR;
}


/**
 * @fn PJUGADOR jugador_ins( PJUGADOR pJ, BOOLEAN esjugador )
 * Crea un jugador y lo inicializa
 * @param pJ Jugador
 * @param esjugador Si es un jugador o un 'RealOnly'
 * @returns ptr al jugador creado
 */
PJUGADOR jugador_ins( PJUGADOR pJ, BOOLEAN esjugador )
{
	int numjug;
	PJUGADOR newJ;

	assert( pJ );

	if( esjugador && jugador_numjug_libre( &numjug) != TEG_STATUS_SUCCESS )
		return NULL;

	newJ = (PJUGADOR) malloc( sizeof(JUGADOR) );
	if( newJ==NULL)
		return NULL;

	pJ->numjug = -1;
	pJ->color = -1;
	memmove( newJ, pJ, sizeof(JUGADOR));
	jugador_initplayer( newJ );
	InitializeListHead( &newJ->next );

	newJ->jugador = esjugador;
	newJ->estado = JUG_ESTADO_CONNECTED;

	if( esjugador ) {
		newJ->numjug = numjug;
		g_juego.jugadores++;
	}

	g_juego.conecciones++;
	InsertTailList( &g_list_jugador, (PLIST_ENTRY) newJ );

	con_text_out(M_INF,_("Player %s(%d) is connected\n"),newJ->nombre,newJ->numjug);
	return newJ;
}

/**
 * @fn TEG_STATUS jugador_flush()
 * Borra todos los jugadores
 */
TEG_STATUS jugador_flush()
{
	PLIST_ENTRY tmp;

	while( !IsListEmpty( &g_list_jugador ) ) {
		tmp = RemoveHeadList( &g_list_jugador );
		if( ((PJUGADOR)tmp)->fd > 0 )
			close( ((PJUGADOR)tmp)->fd);
		con_text_out(M_INF,("Deleting %s\n"),((PJUGADOR)tmp)->nombre);
		free( tmp );
	}
	g_juego.conecciones = 0;
	g_juego.jugadores = 0;
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS jugador_del_soft( PJUGADOR pJ )
 * Puts a player in a GAME OVER state
 */
TEG_STATUS jugador_del_soft( PJUGADOR pJ )
{
	PJUGADOR pJ_new;
	int p;
	int j=0;
	int real_j;

	assert( pJ );
	assert( pJ->jugador );
	assert( pJ->estado >= JUG_ESTADO_START );

	pJ->estado = JUG_ESTADO_GAMEOVER;

	g_juego.playing--;

	if( JUEGO_EMPEZADO && g_juego.playing == 0 ) {

		con_text_out(M_INF,_("Game without players. Initializing another game.\n"));
		juego_new();

	} else if( g_juego.playing == 1 && JUEGO_EMPEZADO ) {
		PLIST_ENTRY l = g_list_jugador.Flink;
		PJUGADOR pJ2;

		while( !IsListEmpty( &g_list_jugador ) && (l != &g_list_jugador) ) {
			pJ2 = (PJUGADOR) l;
			if( pJ2->jugador && pJ2->numjug!=pJ->numjug && pJ2->estado >= JUG_ESTADO_START ) {
				netall_printf( TOKEN_WINNER"=%d,%d\n",pJ2->numjug,pJ2->objetivo );
				con_text_out(M_INF,_("Game with one player. Player %s(%d) is the winner\n"),pJ2->nombre,pJ2->numjug);
				break;
			}
			l = LIST_NEXT(l);
		}

		juego_new();
	} else {
		for(p=0;p<PAISES_CANT;p++) {
			if( g_paises[p].numjug == pJ->numjug ) {
				jugador_from_indice(j, &real_j );
				if( jugador_whois(real_j, &pJ_new)!=TEG_STATUS_SUCCESS )
					continue;
				g_paises[p].numjug = real_j;
				g_paises[p].ejercitos = 1;	/* solo le dejo un solo ejercito */
				InsertTailList( &pJ_new->paises, (PLIST_ENTRY) &g_paises[p] );
				pJ_new->tot_paises++;
				pJ_new->tot_ejercitos += g_paises[p].ejercitos;
				j = (++j % g_juego.playing);
			}
		}

		/* si el jugador tenia el turno, lo tiene que pasar al sig*/
		if( g_juego.turno &&  g_juego.turno->numjug == pJ->numjug ) {
			if( pJ->estado <= JUG_ESTADO_POSTFICHAS ) {
				fichas_next();
			} else if( pJ->estado <= JUG_ESTADO_POSTFICHAS2 ) {
				fichas2_next();
			} else if ( pJ->estado <= JUG_ESTADO_POSTFICHASC ) {
				fichasc_next();
			} else { /* tenia el turno */
				turno_next();
			}
		}

		/* si el jugador empezo el turno, digo que lo empezo el anterior */
		if( g_juego.empieza_turno && g_juego.empieza_turno->numjug == pJ->numjug ) {
			turno_2prevplayer( &g_juego.empieza_turno );
		}
	}

	jugador_initplayer( pJ );

	return TEG_STATUS_SUCCESS;
}


/**
 * @fn TEG_STATUS jugador_del( PJUGADOR pJ )
 * Deletes a player
 * @param pJ Jugador a borrar
 */
TEG_STATUS jugador_del_hard( PJUGADOR pJ )
{
	PLIST_ENTRY l = (PLIST_ENTRY) pJ;

	assert( pJ );

	g_juego.conecciones--;

	/* la quito, ahora, pero la frito luego */
	l = RemoveHeadList( l->Blink );

	if( pJ->jugador ) {

		con_text_out(M_INF,_("Player %s(%d) quit the game\n"),pJ->nombre,pJ->numjug);
		netall_printf( TOKEN_EXIT"=%d\n",pJ->numjug );

		if( pJ->estado >= JUG_ESTADO_START )
			jugador_del_soft( pJ );

		color_del( pJ->color );
		g_juego.jugadores--;
	}

	/* la frito */
	free( l );

	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS jugador_from_indice( int j, int *real_j )
 * Dado un indice de jugador [0...TEG_MAX_PLAYERS] devulve el numjug asociado
 */
TEG_STATUS jugador_from_indice( int j, int *real_j )
{
	PLIST_ENTRY l = g_list_jugador.Flink;
	PJUGADOR pJ;
	int i=0;

	while( !IsListEmpty( &g_list_jugador ) && (l != &g_list_jugador) ) {
		pJ = (PJUGADOR) l;
		if( pJ->jugador && pJ->estado>=JUG_ESTADO_HABILITADO ) {
			if( j == i ) {
				*real_j = pJ->numjug;
				return TEG_STATUS_SUCCESS;
			}
			i++;
		}
		l = LIST_NEXT(l);
	}
	return TEG_STATUS_PLAYERNOTFOUND;
}

/**
 * @fn TEG_STATUS jugador_asignarpais( int numjug, PPAIS p)
 * Asigna un pais a un jugador
 * @param numjug jugador que tendra un nuevo pais
 * @param p Pais que va a ser asignado
 */
TEG_STATUS jugador_asignarpais( int numjug, PPAIS p)
{
	PJUGADOR pJ;

	if( jugador_whois( numjug, &pJ) != TEG_STATUS_SUCCESS )
		return TEG_STATUS_PLAYERNOTFOUND;

	InsertTailList( &pJ->paises, (PLIST_ENTRY) p );
	p->numjug = numjug;
	pJ->tot_paises++;
	pJ->tot_ejercitos++;		/* cada pais viene con un ejercito */
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS jugador_whoisfd( int fd, PJUGADOR *j)
 * Dado un filedescriptor devuelve el jugador que es dueo
 * @param fd filedescriptor del jugador
 * @param j Tendra el jugador
 */
TEG_STATUS jugador_whoisfd( int fd, PJUGADOR *j)
{
	PLIST_ENTRY l = g_list_jugador.Flink;
	PJUGADOR i;

	while( !IsListEmpty( &g_list_jugador ) && (l != &g_list_jugador) ) {
		i = (PJUGADOR) l;
		if( i->fd == fd) {
			*j = i;
			return TEG_STATUS_SUCCESS;
		}
		l = LIST_NEXT(l);
	}
	return TEG_STATUS_PLAYERNOTFOUND;
}

/*
 * TODO: This should be a macro
 */
BOOLEAN jugador_esta_xxx( int fd, int condicion, int strict )
{
	PJUGADOR pJ;

	if( jugador_whoisfd( fd, &pJ) == TEG_STATUS_SUCCESS ) {
		if(strict)
			return (pJ->estado == condicion );
		else
			return (pJ->estado >= condicion );
	} else
		return FALSE;
}

BOOLEAN jugador_esta_xxx_plus( int fd, int condicion, int strict, PJUGADOR *j )
{
	if( jugador_whoisfd( fd, j) == TEG_STATUS_SUCCESS ) {
		if(strict)
			return ((*j)->estado == condicion );
		else
			return ((*j)->estado >= condicion );
	} else
		return FALSE;
}

/**
 * @fn TEG_STATUS jugador_listar_paises( PJUGADOR pJ, int *paises )
 * Dice los paises agrupados por cont que tiene un jugador
 * @param pJ Jugador
 * @param paises Array con los paises por continente que tiene
 * @returns Si fue SUCCESS o no
 */
TEG_STATUS jugador_listar_paises( PJUGADOR pJ, int *paises )
{
	PLIST_ENTRY list;
	PPAIS pP;

	assert( pJ );
	assert( paises );


	list = pJ->paises.Flink;

	while( !IsListEmpty( &pJ->paises ) && (list != &pJ->paises ) ) {
		pP = (PPAIS) list;

		paises[ pP->continente ]++;

		list = LIST_NEXT( list );
	}
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS jugador_listar_conts( PJUGADOR pJ, unsigned long *ret )
 * Dice los continentes que tiene un jugador
 * @param pJ Jugador
 * @param ret Continentes construidos con 'or'
 * @returns Si fue SUCCESS o no
 */
TEG_STATUS jugador_listar_conts( PJUGADOR pJ, unsigned long *ret )
{
	int paises[CONT_CANT];
	int i;

	assert( pJ );
	assert( ret );


	memset( paises, 0, sizeof(paises) );


	if( jugador_listar_paises( pJ, paises ) != TEG_STATUS_SUCCESS ) {
		free( paises );
		return TEG_STATUS_ERROR;
	}

	*ret = 0;

	for(i=0;i< CONT_CANT; i++) {
		if( paises[i] == g_conts[i].cant_paises )
			*ret |= 1 << i;
	}

	return TEG_STATUS_SUCCESS;
}



/**
 * @fn TEG_STATUS jugador_clear_turn( PJUGADOR j )
 * Inicializa algunos valores para que en el sig. turno ando todo bien.
 * @param j Jugador
 * @return Si salio bien, devuelve TEG_STATUS_SUCCESS
 */
TEG_STATUS jugador_clear_turn( PJUGADOR pJ )
{
	int i;

	assert( pJ );

	pJ->turno_conq = 0;
	pJ->pais_src = -1;
	pJ->pais_dst = -1;
	pJ->estado =  JUG_ESTADO_IDLE;

	/* limpiar los reagrupes que pudo haber hecho */
	for(i=0;i<PAISES_CANT;i++) {
		if( g_paises[i].numjug == pJ->numjug )
			g_paises[i].ejer_reagrupe = 0;
	}

	return TEG_STATUS_SUCCESS;
}
/**
 * @fn int jugador_fichasc_cant( PJUGADOR pJ )
 * Dice la cant de paises que puede poner un jugador segun la cant de paises
 */
int jugador_fichasc_cant( PJUGADOR pJ )
{
	assert( pJ );

	if( pJ->tot_paises <= 6 )
		return 3;
	else
		return pJ->tot_paises/2;
}

/**
 * @fn TEG_STATUS jugador_all_set_status( JUG_ESTADO estado )
 * Pone a todos los jugadores en un cierto estado.
 * @param estado Estado en el que se pone a todos los jugadores
 */
TEG_STATUS jugador_all_set_status( JUG_ESTADO estado )
{
	PLIST_ENTRY l = g_list_jugador.Flink;
	PJUGADOR pJ;

	while( !IsListEmpty( &g_list_jugador ) && (l != &g_list_jugador) ) {
		pJ = (PJUGADOR) l;
		if( pJ->jugador && pJ->estado >= JUG_ESTADO_GAMEOVER ) {
			pJ->estado = estado;
		}
		l = LIST_NEXT(l);
	}
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS jugador_map( jug_map_func func )
 * Map para jugador
 */
TEG_STATUS jugador_map( jug_map_func func )
{
	PLIST_ENTRY l = g_list_jugador.Flink;
	PJUGADOR pJ;

	assert(func);

	while( !IsListEmpty( &g_list_jugador ) && (l != &g_list_jugador) ) {
		pJ = (PJUGADOR) l;
		if( pJ->jugador ) {
			(func)(pJ);
		}
		l = LIST_NEXT(l);
	}
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS jugador_chequear_perdio( PJUGADOR pJ )
 * Se fija si perdio este jugador
 * @param pJ Jugador a chequear
 * @return TEG_STATUS_GAMEOVER si perdio
 */
TEG_STATUS jugador_chequear_perdio( PJUGADOR pJ )
{
	assert( pJ );

	if( pJ->tot_paises <= 0 ) {
		return TEG_STATUS_GAMEOVER;
	}

	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS jugador_poner_perdio( PJUGADOR pJ )
 * Pone al jugador pJ en estado 'perdio'
 * @param pJ Jugador a poner es estado 'perdio'
 */
TEG_STATUS jugador_poner_perdio( PJUGADOR pJ )
{
	assert( pJ );
	assert( pJ->jugador == TRUE );

	pJ->estado = JUG_ESTADO_GAMEOVER;

	if( g_juego.empieza_turno && g_juego.empieza_turno->numjug == pJ->numjug )
		turno_2prevplayer( &g_juego.empieza_turno );

	return TEG_STATUS_SUCCESS;
}
