/*	$Id: play.c,v 1.74 2001/06/07 03:39:54 riq Exp $	*/
/* Tenes Empanadas Graciela
 *
 * Copyright (C) 2000 Ricardo Quesada
 *
 * Author: Ricardo Calixto Quesada <rquesada@core-sdi.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
/**
 * @file play.c
 * Maneja los mensajes que llegan
 */

#include <unistd.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "server.h"
#include "tegggz.h"

#undef DEBUG_PLAY

#ifdef DEBUG_PLAY
# define PLAY_DEBUG(x...) PDEBUG(x)
# define STATIC
#else
# define PLAY_DEBUG(x...)
# define STATIC static
#endif


STATIC TEG_STATUS token_status();
STATIC TEG_STATUS token_test();
STATIC TEG_STATUS token_sversion();
STATIC TEG_STATUS token_pversion();
STATIC TEG_STATUS token_playerid();
STATIC TEG_STATUS token_help();
STATIC TEG_STATUS token_paises();
STATIC TEG_STATUS token_fichas();
STATIC TEG_STATUS token_fichas2();
STATIC TEG_STATUS token_fichasc();
STATIC TEG_STATUS token_ataque();
STATIC TEG_STATUS token_qumm();
STATIC TEG_STATUS token_rem();
STATIC TEG_STATUS token_message();
STATIC TEG_STATUS token_tropas();
STATIC TEG_STATUS token_turno();
STATIC TEG_STATUS token_tarjeta();
STATIC TEG_STATUS token_reagrupe();
STATIC TEG_STATUS token_canje();
STATIC TEG_STATUS token_ejer2();
STATIC TEG_STATUS token_objetivo();
STATIC TEG_STATUS token_color();
STATIC TEG_STATUS token_loque();
STATIC TEG_STATUS token_echo();
STATIC TEG_STATUS token_surrender();
STATIC TEG_STATUS token_set();

struct {
	char *label;
	TEG_STATUS (*func) ();
	char *help;
} tokens[] = {
	{ TOKEN_START,		token_start,	N_("to start the game") },
	{ TOKEN_STATUS,		token_status,	N_("shows the status of the players") },
	{ TOKEN_MESSAGE,	token_message,	N_("to send a message") },
	{ TOKEN_EXIT,		token_exit,	N_("to exit the game") },
	{ TOKEN_TEST,		token_test,	N_("internal use. Dont use it.") },
	{ TOKEN_CVERSION,	NULL,		N_("client version") },
	{ TOKEN_SVERSION,	token_sversion,	N_("server version") },
	{ TOKEN_PVERSION,	token_pversion,	N_("protocol version") },
	{ TOKEN_PLAYERID,	token_playerid,	N_("to register as a player") },
	{ TOKEN_HELP,		token_help,	N_("to ask for help") },
	{ TOKEN_REM,		token_rem,	N_("to comment a command") },
	{ TOKEN_QUMM,		token_qumm,	N_("It makes you more happy") },
	{ TOKEN_PAISES,		token_paises,	N_("It shows info about the countries") },
	{ TOKEN_FICHAS,		token_fichas,	N_("to place the initials 5 armies") },
	{ TOKEN_FICHAS2,	token_fichas2,	N_("to place the initials 3 armies") },
	{ TOKEN_FICHASC,	token_fichasc,	N_("to place the armies after a turn have finished") },
	{ TOKEN_ATAQUE,		token_ataque,	N_("to attack a country") },
	{ TOKEN_TROPAS,		token_tropas,	N_("to send armies to a conquered country") },
	{ TOKEN_TARJETA,	token_tarjeta,	N_("to pick up a country-card") },
	{ TOKEN_REAGRUPE,	token_reagrupe,	N_("to reorder your armies") },
	{ TOKEN_TURNO,		token_turno,	N_("to finish your turn") },
	{ TOKEN_CANJE,		token_canje,	N_("to exchange your cards for armies") },
	{ TOKEN_EJER2,		token_ejer2,	N_("to place 2 armies in a country. You must have that card") },
	{ TOKEN_OBJETIVO,	token_objetivo,	N_("request a mission") },
	{ TOKEN_COLOR,		token_color,	N_("to select a color") },
	{ TOKEN_LOQUE,		token_loque,	N_("to remind me what to do") },
	{ TOKEN_ECHO,		token_echo,	N_("to set an async callback") },
	{ TOKEN_SURRENDER,	token_surrender,N_("to surrender") },
	{ TOKEN_SET,		token_set,	N_("to set options") },
};
#define	NTOKENS  (sizeof(tokens)/sizeof(tokens[0]))


/**
 * @fn STATIC TEG_STATUS token_set( int fd, char *str )
 * Sets a server option
 */
STATIC TEG_STATUS token_set( int fd, char *str )
{
	if( strlen(str)==0 )
		goto error;

	if( option_parse(fd, str) == TEG_STATUS_SUCCESS )
		return TEG_STATUS_SUCCESS;

error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_SET"\n");
	return TEG_STATUS_PARSEERROR;
}

/**
 * @fn STATIC TEG_STATUS token_echo( int fd, char *msg )
 * Le envia al jugador el mensaje que pidio. Util como callback
 */
STATIC TEG_STATUS token_echo( int fd, char *msg )
{
	PJUGADOR pJ;
	if( jugador_whoisfd(fd, &pJ )!=TEG_STATUS_SUCCESS || strlen(msg)==0 )
		goto error;

	net_printf(fd,"%s\n",msg);
	return TEG_STATUS_SUCCESS;

error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_ECHO"\n");
	return TEG_STATUS_PARSEERROR;
}

/**
 * @fn STATIC TEG_STATUS token_loque( int fd )
 * Le envia al jugador que es lo que tiene que hacer. Util como callback.
 * TODO: terminar de implementar las demas 'loque'
 */
STATIC TEG_STATUS token_loque( int fd )
{
	PJUGADOR pJ;
	PLAY_DEBUG("token_loque()");
	if( jugador_whoisfd( fd, &pJ ) != TEG_STATUS_SUCCESS ) {
		goto error;
	}

	switch( pJ->estado ) {
	case JUG_ESTADO_FICHAS:
		net_printf( fd, TOKEN_FICHAS"=%d,%d\n",g_juego.turno->numjug,g_juego.fichas);
		break;
	case JUG_ESTADO_FICHAS2:
		net_printf( fd, TOKEN_FICHAS2"=%d,%d\n",g_juego.turno->numjug,g_juego.fichas);
		break;
	case JUG_ESTADO_FICHASC:
	case JUG_ESTADO_CANJE:
		aux_token_fichasc();
		break;
	case JUG_ESTADO_ATAQUE:
	case JUG_ESTADO_TROPAS:
		net_printf( fd, TOKEN_TURNO"=%d\n",g_juego.turno->numjug );
		break;
	default:
		break;
	}

	return TEG_STATUS_SUCCESS;
error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_LOQUE"\n");
	return TEG_STATUS_PARSEERROR;
}

/**
 * @fn STATIC TEG_STATUS token_color( int fd, char *str )
 * Le asigna un color a un jugador
 * @param fd FileDescriptor del jugador
 * @param str color deseado
 */
STATIC TEG_STATUS token_color( int fd, char *str )
{
	int color;
	int a;
	PJUGADOR pJ;
	PLAY_DEBUG("token_color()");

	if( jugador_whoisfd( fd, &pJ ) != TEG_STATUS_SUCCESS ) {
		goto error;
	}

	if(strlen(str)==0)
		goto error;

	if( pJ->estado != JUG_ESTADO_CONNECTED )
		goto error;

	if( pJ->jugador == FALSE )
		goto error;

	a = atoi( str );
	if (a < 0 ||  a >= TEG_MAX_PLAYERS )
		goto error;

	color = a;
	if ( color_libre( &color )  == FALSE )
		goto error;

	pJ->estado = JUG_ESTADO_HABILITADO;
	pJ->color = color;
	con_text_out(M_INF,_("Player %s(%d) has color %s\n"),pJ->nombre,pJ->numjug,_(g_colores[color]));

	netall_printf( TOKEN_NEWPLAYER"=%s,%d,%d\n", pJ->nombre, pJ->numjug, pJ->color );
	return TEG_STATUS_SUCCESS;
error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_COLOR"\n");
	return TEG_STATUS_PARSEERROR;
}


/**
 * @fn STATIC TEG_STATUS token_objetivo( int fd )
 * El jugador pide un objetivo
 * @param fd FileDescriptor del jugador
 */
STATIC TEG_STATUS token_objetivo( int fd )
{
	PJUGADOR pJ;
	PLAY_DEBUG("token_objetivo()");

	if( jugador_whoisfd( fd, &pJ ) != TEG_STATUS_SUCCESS ) {
		goto error;
	}

	if( pJ->estado < JUG_ESTADO_START ) {
		goto error;
	}

	if( objetivo_asignar( pJ ) != TEG_STATUS_SUCCESS )
		goto error;

	net_printf( fd, TOKEN_OBJETIVO"=%d\n",pJ->objetivo);
	return TEG_STATUS_SUCCESS;
error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_OBJETIVO"\n");
	return TEG_STATUS_PARSEERROR;
}

/**
 * @fn STATIC TEG_STATUS token_turno( int fd )
 * Llamado cuando jugador termino el turno y lo pasa al siguiente
 * @param fd FileDescriptor del jugador
 */
STATIC TEG_STATUS token_turno( int fd )
{
	PJUGADOR pJ;
	PLAY_DEBUG("token_turno()");

	if( jugador_whoisfd( fd, &pJ ) != TEG_STATUS_SUCCESS ) {
		goto error;
	}

	if( pJ->estado < JUG_ESTADO_TURNOSTART || pJ-> estado > JUG_ESTADO_TURNOEND )
		goto error;

	pJ->estado = JUG_ESTADO_IDLE;

	/* turno al siguiente */
	turno_end( pJ );
	turno_next();

	return TEG_STATUS_SUCCESS;
error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_TURNO"\n");
	return TEG_STATUS_PARSEERROR;
}

/**
 * @fn STATIC TEG_STATUS token_message( int fd, char *msg )
 * Envia un mensaje a todos los jugadores. Un RO la puede usar
 * @param fd FileDescriptor del jugador
 * @param msg Mensaje a enviar
 */
STATIC TEG_STATUS token_message( int fd, char *msg )
{
	PJUGADOR j;
	PLAY_DEBUG("token_message()\n");

	if( jugador_whoisfd(fd, &j )!=TEG_STATUS_SUCCESS || strlen(msg)==0 )
		goto error;

	if( j->jugador )
		netall_printf(TOKEN_MESSAGE"=%s,%d,%s\n",j->nombre,j->numjug,msg);
	else
		netall_printf(TOKEN_MESSAGE"=observer-%s,%d,%s\n",j->nombre,j->numjug,msg);
	return TEG_STATUS_SUCCESS;
error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_MESSAGE"\n");
	return TEG_STATUS_PARSEERROR;
}

/**
 * @fn STATIC TEG_STATUS token_rem( int fd )
 * funcion REM. No sirve para nada.
 * @param fd FileDescriptor del jugador
 */
STATIC TEG_STATUS token_rem( int fd )
{
	PLAY_DEBUG("token_rem()\n");

	if( !JUGADOR_HABILITADO(fd) )
		goto error;

	net_printf(fd,TOKEN_REM"=%s\n",_("Para que me envias un rem?"));
	return TEG_STATUS_SUCCESS;
error:
	return TEG_STATUS_PARSEERROR;
}

/**
 * @fn STATIC TEG_STATUS token_qumm( int fd, char *str )
 * funcion para "Quiero Un Mundo Mejor"
 * @param fd FileDescriptor del jugador
 * @param str Parametros de qumm
 */
STATIC TEG_STATUS token_qumm( int fd, char *str )
{
	PLAY_DEBUG("token_qumm()\n");

	if( !JUGADOR_HABILITADO(fd) )
		goto error;

	net_printf(fd,TOKEN_REM"=%s\n",_("Yo tambien quiero un mundo mejor!"));
	return TEG_STATUS_SUCCESS;
error:
	return TEG_STATUS_PARSEERROR;
}

/**
 * @fn STATIC TEG_STATUS token_playerid( int fd, char *str )
 * Crea un jugador.
 * @param fd File descriptor
 * @param str Contiene 'nombre,modo'. Si esta bien formado devuelve 'nombre,numjug'
 */
STATIC TEG_STATUS token_playerid( int fd, char *str )
{
	PARSER p;
	DELIM igualador={ ':', ':', ':' };
	DELIM separador={ ',', ',', ',' };
	JUGADOR j, *pJ;
	char c[TEG_MAX_PLAYERS];
	char colores[100];
	int i;

	PLAY_DEBUG("token_playerid()\n");

	/* si existe entonces da error, porque no tiene que existir */
	if( jugador_whoisfd(fd, &pJ ) == TEG_STATUS_SUCCESS )
		goto error;

	if( strlen(str)==0 )
		goto error;

	p.igualador = &igualador;
	p.separador = &separador;
	p.data = str;

	memset( &j, 0, sizeof(JUGADOR));

	/* averigua el nombre */
	if( parser_call( &p ) && p.hay_otro ) {
#ifdef WITH_GGZ
		if(g_server.with_ggz) {
			if( tegggz_find_ggzname(fd,j.nombre,sizeof(j.nombre)-1) != TEG_STATUS_SUCCESS ) {
				strncpy( j.nombre, p.token, sizeof(j.nombre)-1 );
			}
		} else
#endif /*WITH_GGZ */
			strncpy( j.nombre, p.token, sizeof(j.nombre)-1 );
		j.nombre[sizeof(j.nombre)-1]=0;
		aux_depurar_nombre(j.nombre);
	} else goto error;

	if( parser_call( &p ) && !p.hay_otro ) {
		j.jugador = atoi( p.token );		
	} else goto error;


	if( j.jugador ) {
		if( JUEGO_EMPEZADO ) {
			net_printf(fd,TOKEN_SERVERFULL "\n");
			return TEG_STATUS_CONNCLOSED;
		}
		pJ = jugador_ins_player( &j );
	} else
		pJ = jugador_ins_ro( &j );


	if( pJ == NULL ) {
		net_printf(fd,TOKEN_SERVERFULL "\n");
		return TEG_STATUS_CONNCLOSED;
	}

	pJ->fd = fd;

	aux_find_inaddr( pJ );

	colores_libres( c );
	colores[0] = 0;
	for(i=0;i<TEG_MAX_PLAYERS;i++) {
		char buf[100];
		sprintf( buf, ",%d",c[i] );
		strcat( colores, buf );
	}

	net_printf(fd,TOKEN_PLAYERID"=%s,%d%s\n",pJ->nombre,pJ->numjug,colores);
	return TEG_STATUS_SUCCESS;
error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_PLAYERID"\n");
	return TEG_STATUS_PARSEERROR;
}

/**
 * @fn STATIC TEG_STATUS token_fichas( int fd, char *str )
 * parsea las fichas que pone por 1ra vez el jugador
 * @param fd file descriptor del jugador
 * @param str datos que envia el jugador
 * @return TEG_STATUS_SUCCESS si todo salio bien
 */
STATIC TEG_STATUS token_fichas( int fd, char *str )
{
	PLAY_DEBUG("token_fichas()\n");

	if( !JUGADOR_FICHAS(fd) )
		goto error;

	if( aux_token_fichas( fd, str, g_juego.fichas, 0) == TEG_STATUS_SUCCESS ) {

		g_juego.turno->estado = JUG_ESTADO_POSTFICHAS;

		fichas_next();
		return TEG_STATUS_SUCCESS;
	}

error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_FICHAS"\n");
	return TEG_STATUS_PARSEERROR;
}


/**
 * @fn STATIC TEG_STATUS token_fichas2( int fd, char *str )
 * parsea las fichas que pone por 2da vez el jugador
 * @param fd file descriptor del jugador
 * @param str datos que envia el jugador
 * @return TEG_STATUS_SUCCESS si todo salio bien
 */
STATIC TEG_STATUS token_fichas2( int fd, char *str )
{
	PLAY_DEBUG("token_fichas2()\n");

	if( !JUGADOR_FICHAS2(fd) )
		goto error;
	
	if( aux_token_fichas( fd, str, g_juego.fichas2, 0) == TEG_STATUS_SUCCESS ) {

		g_juego.turno->estado = JUG_ESTADO_POSTFICHAS2;

		fichas2_next();
		return TEG_STATUS_SUCCESS;
	}
error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_FICHAS2"\n");
	return TEG_STATUS_PARSEERROR;
}

/**
 * @fn STATIC TEG_STATUS token_fichasc( int fd, char *str )
 * parsea las fichas de contienente y pais
 * @param fd file descriptor del jugador
 * @param str datos que envia el jugador
 * @return TEG_STATUS_SUCCESS si todo salio bien
 */
STATIC TEG_STATUS token_fichasc( int fd, char *str )
{
	unsigned long conts;
	PJUGADOR pJ;
	int maximo;
	PLAY_DEBUG("token_fichasc()\n");

	if( jugador_whoisfd( fd, &pJ ) != TEG_STATUS_SUCCESS )
		goto error;

	if( pJ->estado != JUG_ESTADO_FICHASC && pJ->estado != JUG_ESTADO_CANJE )
		goto error;

	if( jugador_listar_conts( pJ, &conts ) != TEG_STATUS_SUCCESS )
		goto error;

	maximo = jugador_fichasc_cant( pJ ) + cont_tot( conts );
	if( pJ->hizo_canje )
		maximo += cuantos_x_canje( pJ->tot_canjes );

	if( aux_token_fichas( fd, str, maximo, conts) == TEG_STATUS_SUCCESS ) {

		pJ->estado = JUG_ESTADO_IDLE;

		fichasc_next();

		pJ->hizo_canje = FALSE;
		return TEG_STATUS_SUCCESS;
	}

error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_FICHASC"\n");
	return TEG_STATUS_PARSEERROR;
}

/**
 * @fn STATIC TEG_STATUS token_ataque( int fd, char *str )
 * Token que envia un jugador que desea atacar a algun pais
 * @param fd File descriptor del jugador
 * @param str string que contiene 'pais_src,pais_dst'
 */
STATIC TEG_STATUS token_ataque( int fd, char *str )
{
	PARSER p;
	DELIM igualador={ ':', ':', ':' };
	DELIM separador={ ',', ',', ',' };
	int src,dst,src_lost,dst_lost;
	char d_src[3],d_dst[3];
	PJUGADOR j,j_dst;
	int conq = 0;
	int tropas = 0;

	PLAY_DEBUG("token_ataque()\n");

	if( strlen(str)==0)
		goto error;

	if( !JUGADOR_ATAQUE_P(fd,&j)) {
		if( JUGADOR_TROPAS_P(fd,&j)) {
			j->estado=JUG_ESTADO_ATAQUE;
			j->pais_src = j->pais_dst = -1;
		} else goto error;
	}

	p.igualador = &igualador;
	p.separador = &separador;
	p.data = str;

	if( parser_call( &p ) && p.hay_otro ) {
		src = atoi( p.token );		
	} else goto error;

	if( parser_call( &p ) && !p.hay_otro ) {
		dst = atoi( p.token );		
	} else goto error;

	if( src >= PAISES_CANT || src < 0 || dst >= PAISES_CANT || dst < 0) {
		goto error;
	}

	if( j->numjug != g_paises[src].numjug || j->numjug == g_paises[dst].numjug )  {
		goto error;
	}

	if( g_paises[src].ejercitos < 2 || !paises_eslimitrofe( src, dst) ) {
		goto error;
	}

	if( jugador_whois( g_paises[dst].numjug, &j_dst ) != TEG_STATUS_SUCCESS ){
		goto error;
	}

	if( pactos_ataque( src, dst ) != TEG_STATUS_SUCCESS )
		goto error;

	/* aviso a todos que hay un ataque */
	netall_printf( TOKEN_ATAQUE"=%d,%d\n",src,dst );

	/* so far, atacar... */
	aux_token_atacar( g_paises[src].ejercitos, g_paises[dst].ejercitos, &src_lost, &dst_lost, d_src, d_dst );

	g_paises[src].ejercitos -= src_lost;
	g_paises[dst].ejercitos -= dst_lost;
	j->tot_ejercitos -= src_lost;
	j_dst->tot_ejercitos -= dst_lost;

	/* conquisto el pais */
	if( g_paises[dst].ejercitos == 0) {
		PLIST_ENTRY l;

		conq = 1;

		j->turno_conq++;
		j->tot_paises++;

		
		g_paises[dst].numjug = j->numjug;

		g_paises[dst].ejercitos++;		/* se pasa automaticamente */
		g_paises[src].ejercitos--;		/* un ejercito */

		tropas = g_paises[src].ejercitos - 1;	/* cantidad que se pueden pasar */
		if( tropas > 2 )			/* En verdad son 3, pero ya se le paso 1 */
			tropas =2;

		j->estado = JUG_ESTADO_TROPAS;
		j->pais_src = src;
		j->pais_dst = dst;
	
		j_dst->tot_paises--;

		l= RemoveHeadList( g_paises[dst].next.Blink );
		InsertTailList( &j->paises, l);

	}
	/* aviso a todos como fue el resultado del ataque */
	netall_printf(TOKEN_DADOS"=%d,%d,%d,%d,%d,%d,%d,%d;"TOKEN_PAIS"=%d,%d,%d;"TOKEN_PAIS"=%d,%d,%d\n",
			src,d_src[0],d_src[1],d_src[2], dst,d_dst[0],d_dst[1],d_dst[2],
			src,g_paises[src].numjug,g_paises[src].ejercitos,
			dst,g_paises[dst].numjug,g_paises[dst].ejercitos
			);

	if( conq == 1 ) {

		/* Me fijo si 'dst' perdio */
		if( jugador_chequear_perdio( j_dst ) == TEG_STATUS_GAMEOVER ) {
			con_text_out(M_INF,_("Player %s(%d) lost the game\n"),j_dst->nombre,j_dst->numjug);
			netall_printf( TOKEN_LOST"=%d\n",j_dst->numjug );
			jugador_poner_perdio(j_dst);
		}

		/* Me fijo si 'src' gano */
		if( objetivo_chequear( j ) == TEG_STATUS_GAMEOVER ) {
			con_text_out(M_INF,_("Player %s(%d) is the winner! Game Over\n"),j->nombre,j->numjug);
			netall_printf( TOKEN_WINNER"=%d,%d\n",j->numjug,j->objetivo );
			juego_new();
			return TEG_STATUS_SUCCESS;
		}

		net_printf(fd,TOKEN_TROPAS"=%d,%d,%d\n",src,dst,tropas);
	}

	return TEG_STATUS_SUCCESS;
error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_ATAQUE"\n");
	return TEG_STATUS_PARSEERROR;
}

/**
 * @fn STATIC TEG_STATUS token_tropas( int fd, char *str )
 * Pasa ejercitos al pais conquistado
 */
STATIC TEG_STATUS token_tropas( int fd, char *str )
{
	PARSER p;
	DELIM igualador={ ':', ':', ':' };
	DELIM separador={ ',', ',', ',' };
	int src,dst,cant;
	PJUGADOR j;

	PLAY_DEBUG("token_tropas()\n");

	if( !JUGADOR_TROPAS_P(fd,&j) || strlen(str)==0 )
		goto error;

	p.igualador = &igualador;
	p.separador = &separador;
	p.data = str;

	if( parser_call( &p ) && p.hay_otro ) {
		src = atoi( p.token );		
	} else goto error;

	if( parser_call( &p ) && p.hay_otro ) {
		dst = atoi( p.token );		
	} else goto error;

	if( parser_call( &p ) && !p.hay_otro ) {
		cant = atoi( p.token );		
	} else goto error;

	if( src >= PAISES_CANT || src < 0 || dst >= PAISES_CANT || dst < 0) {
		goto error;
	}

	if( src != j->pais_src || dst != j->pais_dst )
		goto error;

	if( cant > g_paises[src].ejercitos-1 || cant > 3)
		goto error;

	if( cant > 0 ) {
		g_paises[src].ejercitos -= cant;
		g_paises[dst].ejercitos += cant;

		j->estado=JUG_ESTADO_ATAQUE;
		j->pais_src = j->pais_dst = -1;

		netall_printf(TOKEN_PAIS"=%d,%d,%d;"TOKEN_PAIS"=%d,%d,%d\n",
				src,g_paises[src].numjug,g_paises[src].ejercitos,
				dst,g_paises[dst].numjug,g_paises[dst].ejercitos
				);
	}


	return TEG_STATUS_SUCCESS;
error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_TROPAS"\n");
	return TEG_STATUS_PARSEERROR;
}

/**
 * @fn STATIC TEG_STATUS token_tarjeta( int fd, char *str )
 * @brief Token usado para sacar una tarjeta luego de terminar el turno.
 * @param fd File descriptor del jugador
 * @param str Parametros recibidos
 */
STATIC TEG_STATUS token_tarjeta( int fd, char *str )
{
	PJUGADOR j;
	PPAIS pP;

	PLAY_DEBUG("token_tarjeta()\n");


	/* Veo si puede sacar una tarjeta... */
	if( jugador_whoisfd( fd, &j ) != TEG_STATUS_SUCCESS )
		goto error;

	if( j->estado < JUG_ESTADO_TURNOSTART || j->estado >= JUG_ESTADO_TARJETA)
		goto error;

	if( j->tot_tarjetas >= TEG_MAX_TARJETAS )
		goto error;

	if( j->turno_conq < 1 )
		goto error;

	if( j->tot_canjes > 3 && j->turno_conq < 2 )
		goto error;


	/* Puede sacar tarjeta */
	j->tot_tarjetas++;
	pP = get_random_pais( tarjeta_es_libre );

	j->estado = JUG_ESTADO_TARJETA;

	tarjeta_sacar( &pP->tarjeta, j->numjug );

	/*
	 * Me fijo si el jugador es dueo del pas que dice la tarjeta. Si es as
	 * le agrego 2 fichas automaticamente como dice el reglamento.
	 */
	if( pP->numjug == j->numjug ) {
		pP->ejercitos += 2;
		j->tot_ejercitos += 2;
		tarjeta_usar( &pP->tarjeta );
		netall_printf(TOKEN_PAIS"=%d,%d,%d\n",pP->id,pP->numjug,pP->ejercitos);
	} 
	net_printf(fd,TOKEN_TARJETA"=%d,%d\n",pP->id,pP->tarjeta.usada);

	return TEG_STATUS_SUCCESS;
error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_TARJETA"\n");
	return TEG_STATUS_ERROR;
}

/**
 * @fn STATIC TEG_STATUS token_ejer2( int fd, char *str )
 * @brief Pone 2 ejercitos en un pais. Se debe tener esa tarjeta para 
 * pueda hacer esto.
 */
STATIC TEG_STATUS token_ejer2( int fd, char *str )
{
	PARSER p;
	DELIM igualador={ ':', ':', ':' };
	DELIM separador={ ',', ',', ',' };
	int pais;
	PJUGADOR j;

	PLAY_DEBUG("token_ejer2()\n");

	if( strlen(str)==0)
		goto error;

	if( jugador_whoisfd( fd, &j ) != TEG_STATUS_SUCCESS )
		goto error;

	if( j->estado != JUG_ESTADO_TARJETA )
		goto error;

	p.igualador = &igualador;
	p.separador = &separador;
	p.data = str;

	if( parser_call( &p ) && !p.hay_otro ) {
		pais = atoi( p.token );		
	} else goto error;

	if( pais >= PAISES_CANT || pais < 0 ) {
		goto error;
	}

	if( tarjeta_es_usada( &g_paises[ pais ].tarjeta ))
		goto error;

	if( g_paises[ pais ].numjug == j->numjug && g_paises[ pais ].tarjeta.numjug == j->numjug ) {
		g_paises[ pais ].ejercitos += 2;
		j->tot_ejercitos += 2;
		tarjeta_usar( &g_paises[ pais ].tarjeta );
		netall_printf(TOKEN_PAIS"=%d,%d,%d\n"
				,pais
				,g_paises[ pais ].numjug
				,g_paises[ pais ].ejercitos);
	} 

	return TEG_STATUS_SUCCESS;
error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_EJER2"\n");
	return TEG_STATUS_PARSEERROR;
}


/**
 * @fn STATIC TEG_STATUS token_canje( int fd )
 * Sirve para cambiar tarjetas por ejercitos
 */
STATIC TEG_STATUS token_canje( int fd, char *str )
{
	PARSER p;
	DELIM igualador={ ':', ':', ':' };
	DELIM separador={ ',', ',', ',' };
	PJUGADOR pJ;
	int t1,t2,t3;
	int canj_ejer;

	PLAY_DEBUG("token_canje()\n");

	if( jugador_whoisfd( fd, &pJ ) != TEG_STATUS_SUCCESS )
		goto error;

	if( pJ->estado != JUG_ESTADO_FICHASC )
		goto error;

	p.igualador = &igualador;
	p.separador = &separador;
	p.data = str;

	if( parser_call( &p ) && p.hay_otro ) {
		t1 = atoi( p.token );		
	} else goto error;

	if( parser_call( &p ) && p.hay_otro ) {
		t2 = atoi( p.token );		
	} else goto error;

	if( parser_call( &p ) && !p.hay_otro ) {
		t3 = atoi( p.token );		
	} else goto error;

	/* se puede hacer el canje ? */
	if( !tarjeta_puedocanje( pJ->numjug, t1, t2, t3 ) )
		goto error;

	pJ->estado = JUG_ESTADO_CANJE;

	pJ->hizo_canje = TRUE;
	pJ->tot_canjes++;
	pJ->tot_tarjetas -= 3;

	canj_ejer = cuantos_x_canje( pJ->tot_canjes );

	/* quitarle las tarjetas al jugador */
	tarjeta_poner( &g_paises[t1].tarjeta );
	tarjeta_poner( &g_paises[t2].tarjeta );
	tarjeta_poner( &g_paises[t2].tarjeta );

	netall_printf(TOKEN_CANJE"=%d,%d,%d,%d,%d\n",
			pJ->numjug,canj_ejer,t1,t2,t3);
	return TEG_STATUS_SUCCESS;
error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_CANJE"\n");
	return TEG_STATUS_PARSEERROR;
}


/**
 * @fn STATIC TEG_STATUS token_reagrupe( int fd, char *str )
 * Token usado para reagrupar los ejercitos.
 * @param fd File descriptor del jugador
 * @param str Parametros recibidos
 */
STATIC TEG_STATUS token_reagrupe( int fd, char *str )
{
	PARSER p;
	DELIM igualador={ ':', ':', ':' };
	DELIM separador={ ',', ',', ',' };
	int src,dst,cant;
	int ejer_disp;
	PJUGADOR pJ;

	PLAY_DEBUG("token_reagrupe()\n");


	if( strlen(str)==0)
		goto error;

	if( jugador_whoisfd( fd, &pJ ) != TEG_STATUS_SUCCESS )
		goto error;

	if( pJ->estado < JUG_ESTADO_TURNOSTART || pJ->estado > JUG_ESTADO_REAGRUPE)
		goto error;

	p.igualador = &igualador;
	p.separador = &separador;
	p.data = str;


	if( parser_call( &p ) && p.hay_otro ) {
		src = atoi( p.token );
	} else goto error;

	if( parser_call( &p ) && p.hay_otro ) {
		dst = atoi( p.token );
	} else goto error;

	if( parser_call( &p ) && !p.hay_otro ) {
		cant = atoi( p.token );
	} else goto error;

	if( src >= PAISES_CANT || src < 0 || dst >= PAISES_CANT || dst < 0 || cant <= 0) {
		goto error;
	}

	if( g_paises[src].numjug != pJ->numjug || g_paises[dst].numjug != pJ->numjug ) {
		goto error;
	}

	if( !paises_eslimitrofe( src, dst ))
		goto error;

	ejer_disp = g_paises[src].ejercitos - g_paises[src].ejer_reagrupe - 1;
	if( cant > ejer_disp )
		goto error;

	pJ->estado = JUG_ESTADO_REAGRUPE;

	g_paises[dst].ejercitos += cant;
	g_paises[dst].ejer_reagrupe += cant;
	g_paises[src].ejercitos -= cant;

	netall_printf(TOKEN_PAIS"=%d,%d,%d;"TOKEN_PAIS"=%d,%d,%d\n",
		src,g_paises[src].numjug,g_paises[src].ejercitos,
		dst,g_paises[dst].numjug,g_paises[dst].ejercitos
	);

	return TEG_STATUS_SUCCESS;
error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_REAGRUPE"\n");
	return TEG_STATUS_PARSEERROR;
}

/**
 * @fn STATIC TEG_STATUS token_status( int fd )
 * Muestra el estado de los jugadores
 */
STATIC TEG_STATUS token_status( int fd )
{
	char strout[PROT_MAX_LEN + PLAYERNAME_MAX_LEN * TEG_MAX_PLAYERS + 200];

	PLAY_DEBUG("token_status()\n");

	if( !JUGADOR_CONNECTED( fd ))
		goto error;

	aux_token_stasta(strout);
	net_printf(fd,TOKEN_STATUS"=%s\n",strout);

	return TEG_STATUS_SUCCESS;

error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_STATUS"\n");
	return TEG_STATUS_PARSEERROR;
}

/**
 * @fn STATIC TEG_STATUS token_paises( int fd, char *str )
 * Dice los paises de un jugador en particular. Un RO la puede usar
 * Si el jugador es -1, da los paises de todo el mundo
 * @param fd filedescriptor del que pide
 * @param str De que jugador se estan pidiendo
 */
STATIC TEG_STATUS token_paises( int fd, char *str )
{
	PARSER p;
	DELIM igualador={ ':', ':', ':' };
	DELIM separador={ ',', ',', ',' };
	int i;
	PJUGADOR pJ;
	char strout[PROT_MAX_LEN];

	PLAY_DEBUG("token_paises()\n");

	if( !JUEGO_EMPEZADO )
		goto error;

	if( strlen(str)==0 )
		goto error;

	p.igualador = &igualador;
	p.separador = &separador;
	p.data = str;

	if( parser_call( &p ) && !p.hay_otro ) {
		i = atoi( p.token );		
	} else
		goto error;

	if( i != -1 && jugador_whois( i, &pJ ) != TEG_STATUS_SUCCESS )
		goto error;

	if(i!=-1) {
		/* ask just for 1 player */
		if( aux_token_paises(pJ,strout,sizeof(strout)-1) !=TEG_STATUS_SUCCESS )
			goto error;
		net_printf(fd,TOKEN_PAISES"=%d/%s\n",pJ->numjug,strout);
	} else  {
		/* ask for all the players */
		PLIST_ENTRY pL = g_list_jugador.Flink;

		while( !IsListEmpty( &g_list_jugador ) && (pL != &g_list_jugador) ) {
			pJ = (PJUGADOR) pL;
			if( pJ->jugador && pJ->estado >= JUG_ESTADO_START) {
				if( aux_token_paises(pJ,strout,sizeof(strout)-1) !=TEG_STATUS_SUCCESS )
					goto error;
				net_printf(fd,TOKEN_PAISES"=%d/%s\n",pJ->numjug,strout);
			}
			pL = LIST_NEXT(pL);
		}
	}
	return TEG_STATUS_SUCCESS;

error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_PAISES"\n");
	return TEG_STATUS_PARSEERROR;
}

/**
 * @fn STATIC TEG_STATUS token_start( int fd )
 * Para empezar un partido
 * @param fd Filedescriptor del que quiere emepezar
 */
TEG_STATUS token_start( int fd )
{
	char strout[PROT_MAX_LEN + PLAYERNAME_MAX_LEN  * TEG_MAX_PLAYERS + 200];
	PJUGADOR pJ;
	PLAY_DEBUG("token_start()\n");

	if( JUEGO_EMPEZADO || g_juego.jugadores < 2 )
		goto error;

	if( g_server.with_console && fd != CONSOLE_FD) {
		if( !JUGADOR_HABILITADO_P(fd,&pJ) || !pJ->jugador )
			goto error;
	}

	JUEGO_EN_EMPEZAR;

	g_juego.playing = g_juego.jugadores;

	con_text_out(M_INF,_("Starting game number: %d with seed: %d\n"),g_juego.gamenumber,g_juego.seed);

	jugador_all_set_status ( JUG_ESTADO_START );
	paises_repartir();

	if(turno_init() != TEG_STATUS_SUCCESS ) {
		con_text_out(M_ERR,_("Error, cant initilize a new turn\n"));
		goto error;
	}

	JUEGO_EN_FICHAS;

	g_juego.turno->estado = JUG_ESTADO_FICHAS;

	aux_token_stasta(strout);

	netall_printf( TOKEN_START"=%s;"TOKEN_MODALIDAD"=%d,%d;"TOKEN_FICHAS"=%d,%d\n"
			,strout			/* jugadores disponibles */
			,g_juego.objetivos	/* se juega con objetivos ? */
			,g_juego.reglas		/* con que reglas ? */
			,g_juego.turno->numjug,	/* quien empiza */
			g_juego.fichas );	/* y cuantas fichas hay que poner */
	return TEG_STATUS_SUCCESS;
error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_START"\n");
	return TEG_STATUS_PARSEERROR;
}

/**
 * @fn STATIC TEG_STATUS token_test( int fd )
 * Usado para hacer test
 */
STATIC TEG_STATUS token_test( int fd )
{
	PLAY_DEBUG("token_test()\n");

	return TEG_STATUS_SUCCESS;
}

/**
 * @fn STATIC TEG_STATUS token_sversion( int fd )
 * Server version
 */
STATIC TEG_STATUS token_sversion( int fd )
{
	PLAY_DEBUG("token_sversion()\n");

	net_printf(fd,TOKEN_SVERSION"=%s %s\n",_("TEG server version "),VERSION);
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn STATIC TEG_STATUS token_pversion( int fd, char *str )
 * Protocol version
 */
STATIC TEG_STATUS token_pversion( int fd, char *str )
{
	PJUGADOR pJ;
	PARSER p;
	DELIM igualador={ ':', ':', ':' };
	DELIM separador={ ',', ',', ',' };
	int hi,lo;

	PLAY_DEBUG("token_pversion()\n");


	if( strlen(str)==0 )
		goto error;

	p.igualador = &igualador;
	p.separador = &separador;
	p.data = str;

	if( parser_call( &p ) && p.hay_otro ) {
		hi = atoi( p.token );		
	} else goto error;

	if( parser_call( &p ) && !p.hay_otro ) {
		lo = atoi( p.token );		
	} else goto error;

	net_printf(fd,TOKEN_PVERSION"=%i,%i\n",PROTOCOL_HIVER,PROTOCOL_LOVER);

	if( hi != PROTOCOL_HIVER ) {
		con_text_out(M_ERR,_("Client with incompatible protocol version (server:%d , client:%d)\n"),PROTOCOL_HIVER,hi);

		if( jugador_whoisfd( fd, &pJ ) == TEG_STATUS_SUCCESS )
			jugador_del_hard( pJ );
		return TEG_STATUS_CONNCLOSED;
	}

	return TEG_STATUS_SUCCESS;

error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_PVERSION"\n");
	return TEG_STATUS_PARSEERROR;
}

/**
 * @fn STATIC TEG_STATUS token_exit( int fd )
 * Salir del TEG
 */
TEG_STATUS token_exit( int fd )
{
	PJUGADOR j;
	PLAY_DEBUG("token_exit\n");

	if( jugador_whoisfd( fd, &j ) == TEG_STATUS_SUCCESS )
		jugador_del_hard( j );
	return TEG_STATUS_CONNCLOSED;
}

/**
 * @fn TEG_STATUS token_surrender( int fd )
 * Puts the player in Game Over state
 */
TEG_STATUS token_surrender( int fd )
{
	PJUGADOR pJ;
	PLAY_DEBUG("token_surrender\n");

	if( jugador_whoisfd( fd, &pJ ) != TEG_STATUS_SUCCESS )
		goto error;

	if( !pJ->jugador )
		goto error;

	if( pJ->estado < JUG_ESTADO_HABILITADO )
		goto error;

	con_text_out(M_INF,_("Player %s(%d) abandoned the game\n"),pJ->nombre,pJ->numjug);

	netall_printf(TOKEN_SURRENDER"=%d\n",pJ->numjug );

	jugador_del_soft( pJ );

	return TEG_STATUS_SUCCESS;
error:
	net_printf(fd,TOKEN_ERROR"="TOKEN_SURRENDER"\n");
	return TEG_STATUS_PARSEERROR;
}

/**
 * @fn STATIC TEG_STATUS token_help ( int fd )
 * Pedir ayuda
 */
STATIC TEG_STATUS token_help ( int fd )
{
	int i;
	for(i=0;i<NTOKENS;i++) {
		if(tokens[i].func)
			net_printf(fd, TOKEN_REM"='%s' %s\n",tokens[i].label,_(tokens[i].help));
	}
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn STATIC TEG_STATUS token_lookup( int fd, PARSER *p )
 * Parsea los tokens
 */
STATIC TEG_STATUS token_lookup( int fd, PARSER *p )
{
	int i;

	for(i = 0; i < NTOKENS; i++) {
		if(strcasecmp( p->token, tokens[i].label )==0 ){
			if (tokens[i].func)
				return( (tokens[i].func)(fd,p->value));
			return TEG_STATUS_TOKENNULL;
		}
	}
	PLAY_DEBUG("Token '%s' no encontrado\n",p->token);
	return TEG_STATUS_TOKENNOTFOUND;
}

/**
 * @fn TEG_STATUS play_teg( int fd )
 * Interpreta lo que tiene el file descriptor
 */
TEG_STATUS play_teg( int fd )
{
	int i,j;
	PARSER p;
	char str[PROT_MAX_LEN];
	DELIM igualador={ '=', '=', '=' };
	DELIM separador={ ';', ';', ';' };

	p.igualador = &igualador;
	p.separador = &separador;

	str[0]=0;

	j=net_readline( fd, str, PROT_MAX_LEN );

	if( j<1 ) {
		PJUGADOR j;
		if( jugador_whoisfd( fd, &j ) == TEG_STATUS_SUCCESS )
			jugador_del_hard( j );
		return TEG_STATUS_CONNCLOSED;
	}
	
	p.data = str;

	do {
		if( (i=parser_call( &p )) ) {
			if( token_lookup( fd,&p ) == TEG_STATUS_CONNCLOSED ) {
				return TEG_STATUS_CONNCLOSED;
			}
		}
	} while( i && p.hay_otro);

	return TEG_STATUS_SUCCESS;
}
