/*	$Id: turno.c,v 1.20 2001/05/28 03:33:04 riq Exp $	*/
/* Tenes Empanadas Graciela
 *
 * Copyright (C) 2000 Ricardo Quesada
 *
 * Author: Ricardo Calixto Quesada <rquesada@core-sdi.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
/**
 * @file turno.c
 * Manejo de turnos
 */

#include <stdio.h>
#include <stdlib.h>
#include "server.h"


#undef DEBUG_TURNO

#ifdef DEBUG_TURNO
# define TURNO_DEBUG(x...) PDEBUG(x)
#else
# define TURNO_DEBUG(x...)
#endif

/**
 * @fn TEG_STATUS turno_2nextplayer( PJUGADOR *j )
 * asigna turno al siguiente
 */
TEG_STATUS turno_2nextplayer( PJUGADOR *j )
{
	PJUGADOR pJ;
	PLIST_ENTRY l = LIST_NEXT( (*j));

	TURNO_DEBUG("Turno viejo de '%s'\n",(*j)->nombre);

	while( !IsListEmpty( (PLIST_ENTRY) (*j) ) && (l != (PLIST_ENTRY)(*j)) ) {
		pJ = (PJUGADOR) l;
		if( (l != &g_list_jugador) && pJ->jugador && pJ->estado >= JUG_ESTADO_START ) {
			(*j) = pJ;
			TURNO_DEBUG("Turno nuevo de '%s'\n",(*j)->nombre);
			return TEG_STATUS_SUCCESS;
		}
		l = LIST_NEXT(l);
	}
	TURNO_DEBUG("XXX\n");
	return TEG_STATUS_PLAYERNOTFOUND;
}

/**
 * @fn TEG_STATUS turno_2prevplayer( PJUGADOR *j )
 * asigna turno al anterior (usado cuando un jugador se fue)
 */
TEG_STATUS turno_2prevplayer( PJUGADOR *ppj )
{
	PJUGADOR pJ;
	PLIST_ENTRY l = LIST_PREV( (*ppj));

	TURNO_DEBUG("Turno viejo de '%s'\n",(*ppj)->nombre);

	while( !IsListEmpty( (PLIST_ENTRY) (*ppj) ) && (l != (PLIST_ENTRY)(*ppj)) ) {
		pJ = (PJUGADOR) l;
		if( (l != &g_list_jugador) && pJ->jugador && pJ->estado >= JUG_ESTADO_START ) {
			(*ppj) = pJ;
			TURNO_DEBUG("Turno nuevo de '%s'\n",(*ppj)->nombre);
			return TEG_STATUS_SUCCESS;
		}
		l = LIST_PREV(l);
	}
	TURNO_DEBUG("XXX\n");
	return TEG_STATUS_PLAYERNOTFOUND;
}

/**
 * @fn TEG_STATUS turno_end( PJUGADOR pJ )
 * Termina el turno del jugador
 * @param pJ jugador que termina su turno
 * @return TEG_STATUS_GAMEOVER si gano el partido
 */
TEG_STATUS turno_end( PJUGADOR pJ )
{
	/* FIXME: Creo que igual este chequeo no sirve, porque se chequea cuando
	 * se conquista un pais. Solo sirve si uno gana sin conquistar algun pais
	 * Puede pasar ???
	 */
	if( objetivo_chequear( g_juego.turno ) == TEG_STATUS_GAMEOVER ) {
		netall_printf( TOKEN_WINNER"=%d,%d\n",g_juego.turno->numjug,g_juego.turno->objetivo);
		juego_new();
		return TEG_STATUS_GAMEOVER;
	}

	jugador_clear_turn( g_juego.turno );
	return TEG_STATUS_SUCCESS;
}

/**
 * @fn void turno_next( void )
 * Pasa el turno de ataque al siguiente jugador
 */
TEG_STATUS turno_next( void )
{

	/* el turno al siguiente */
	turno_2nextplayer( &g_juego.turno );


	/* el que tiene el turno, es el que empezo ? */
	if( g_juego.turno == g_juego.empieza_turno ) {
		/* Se empieza una ronda nueva */
		turno_2nextplayer( &g_juego.empieza_turno );
		g_juego.turno = g_juego.empieza_turno;
		aux_token_fichasc();

	} else {
		g_juego.turno->estado = JUG_ESTADO_ATAQUE;
		netall_printf( TOKEN_TURNO"=%d\n",g_juego.turno->numjug);
	}

	return TEG_STATUS_SUCCESS;
}

/**
 * @fn TEG_STATUS turno_init(void)
 * Inicializa los turnos. Llamar a esta funcion antes de que empiece el juego
 * con los jugadores ya conectados
 */
TEG_STATUS turno_init(void)
{
	int i;
	int real_i;
	PJUGADOR j;

	i = RANDOM_MAX(0,g_juego.playing-1);

	jugador_from_indice( i , &real_i );
	if( jugador_whois( real_i, &j) != TEG_STATUS_SUCCESS )
		return TEG_STATUS_ERROR;

	g_juego.turno = j;
	g_juego.empieza_turno = j;

	return TEG_STATUS_SUCCESS;
}
